@@ -9,7 +9,7 @@ import sys
 
 sys.path[0:0] = [os.path.abspath("..")]
 
-import pymongo
+import pymongo  # noqa
 
 # -- General configuration -----------------------------------------------------
 
@@ -23,6 +23,16 @@ extensions = [
     "sphinx.ext.intersphinx",
 ]
 
+
+# Add optional extensions
+try:
+    import sphinxcontrib.shellcheck  # noqa
+
+    extensions += ["sphinxcontrib.shellcheck"]
+except ImportError:
+    pass
+
+
 # Add any paths that contain templates here, relative to this directory.
 templates_path = ["_templates"]
 
@@ -70,6 +80,13 @@ pygments_style = "sphinx"
 # A list of ignored prefixes for module index sorting.
 # modindex_common_prefix = []
 
+# Options for link checking
+# The anchors on the rendered markdown page are created after the fact,
+# so this link results in a 404.
+linkcheck_ignore = [
+    "https://github.com/mongodb/specifications/blob/master/source/server-discovery-and-monitoring/server-monitoring.rst#requesting-an-immediate-check"
+]
+
 # -- Options for extensions ----------------------------------------------------
 autoclass_content = "init"
 
