@@ -18,6 +18,20 @@
 import datetime
 import warnings
 from collections import OrderedDict, abc
+from typing import (
+    Any,
+    Callable,
+    Dict,
+    List,
+    Mapping,
+    MutableMapping,
+    NoReturn,
+    Optional,
+    Sequence,
+    Tuple,
+    Type,
+    Union,
+)
 from urllib.parse import unquote_plus
 
 from bson import SON
@@ -37,11 +51,11 @@ from pymongo.read_preferences import _MONGOS_MODES, _ServerMode
 from pymongo.server_api import ServerApi
 from pymongo.write_concern import DEFAULT_WRITE_CONCERN, WriteConcern
 
-ORDERED_TYPES = (SON, OrderedDict)
+ORDERED_TYPES: Sequence[Type] = (SON, OrderedDict)
 
 # Defaults until we connect to a server and get updated limits.
 MAX_BSON_SIZE = 16 * (1024**2)
-MAX_MESSAGE_SIZE = 2 * MAX_BSON_SIZE
+MAX_MESSAGE_SIZE: int = 2 * MAX_BSON_SIZE
 MIN_WIRE_VERSION = 0
 MAX_WIRE_VERSION = 0
 MAX_WRITE_BATCH_SIZE = 1000
@@ -49,7 +63,7 @@ MAX_WRITE_BATCH_SIZE = 1000
 # What this version of PyMongo supports.
 MIN_SUPPORTED_SERVER_VERSION = "3.6"
 MIN_SUPPORTED_WIRE_VERSION = 6
-MAX_SUPPORTED_WIRE_VERSION = 14
+MAX_SUPPORTED_WIRE_VERSION = 15
 
 # Frequency to call hello on servers, in seconds.
 HEARTBEAT_FREQUENCY = 10
@@ -86,13 +100,13 @@ MIN_POOL_SIZE = 0
 MAX_CONNECTING = 2
 
 # Default value for maxIdleTimeMS.
-MAX_IDLE_TIME_MS = None
+MAX_IDLE_TIME_MS: Optional[int] = None
 
 # Default value for maxIdleTimeMS in seconds.
-MAX_IDLE_TIME_SEC = None
+MAX_IDLE_TIME_SEC: Optional[int] = None
 
 # Default value for waitQueueTimeoutMS in seconds.
-WAIT_QUEUE_TIMEOUT = None
+WAIT_QUEUE_TIMEOUT: Optional[int] = None
 
 # Default value for localThresholdMS.
 LOCAL_THRESHOLD_MS = 15
@@ -103,11 +117,11 @@ RETRY_WRITES = True
 # Default value for retryReads.
 RETRY_READS = True
 
-# mongod/s 2.6 and above return code 59 when a command doesn't exist.
-COMMAND_NOT_FOUND_CODES = (59,)
+# The error code returned when a command doesn't exist.
+COMMAND_NOT_FOUND_CODES: Sequence[int] = (59,)
 
 # Error codes to ignore if GridFS calls createIndex on a secondary
-UNAUTHORIZED_CODES = (13, 16547, 16548)
+UNAUTHORIZED_CODES: Sequence[int] = (13, 16547, 16548)
 
 # Maximum number of sessions to send in a single endSessions command.
 # From the driver sessions spec.
@@ -117,7 +131,7 @@ _MAX_END_SESSIONS = 10000
 SRV_SERVICE_NAME = "mongodb"
 
 
-def partition_node(node):
+def partition_node(node: str) -> Tuple[str, int]:
     """Split a host:port string into (host, int(port)) pair."""
     host = node
     port = 27017
@@ -129,7 +143,7 @@ def partition_node(node):
     return host, port
 
 
-def clean_node(node):
+def clean_node(node: str) -> Tuple[str, int]:
     """Split and normalize a node name from a hello response."""
     host, port = partition_node(node)
 
@@ -140,7 +154,7 @@ def clean_node(node):
     return host.lower(), port
 
 
-def raise_config_error(key, dummy):
+def raise_config_error(key: str, dummy: Any) -> NoReturn:
     """Raise ConfigurationError with the given key name."""
     raise ConfigurationError("Unknown option %s" % (key,))
 
@@ -155,23 +169,23 @@ _UUID_REPRESENTATIONS = {
 }
 
 
-def validate_boolean(option, value):
+def validate_boolean(option: str, value: Any) -> bool:
     """Validates that 'value' is True or False."""
     if isinstance(value, bool):
         return value
     raise TypeError("%s must be True or False" % (option,))
 
 
-def validate_boolean_or_string(option, value):
+def validate_boolean_or_string(option: str, value: Any) -> bool:
     """Validates that value is True, False, 'true', or 'false'."""
     if isinstance(value, str):
         if value not in ("true", "false"):
-            raise ValueError("The value of %s must be " "'true' or 'false'" % (option,))
+            raise ValueError("The value of %s must be 'true' or 'false'" % (option,))
         return value == "true"
     return validate_boolean(option, value)
 
 
-def validate_integer(option, value):
+def validate_integer(option: str, value: Any) -> int:
     """Validates that 'value' is an integer (or basestring representation)."""
     if isinstance(value, int):
         return value
@@ -179,27 +193,27 @@ def validate_integer(option, value):
         try:
             return int(value)
         except ValueError:
-            raise ValueError("The value of %s must be " "an integer" % (option,))
+            raise ValueError("The value of %s must be an integer" % (option,))
     raise TypeError("Wrong type for %s, value must be an integer" % (option,))
 
 
-def validate_positive_integer(option, value):
+def validate_positive_integer(option: str, value: Any) -> int:
     """Validate that 'value' is a positive integer, which does not include 0."""
     val = validate_integer(option, value)
     if val <= 0:
-        raise ValueError("The value of %s must be " "a positive integer" % (option,))
+        raise ValueError("The value of %s must be a positive integer" % (option,))
     return val
 
 
-def validate_non_negative_integer(option, value):
+def validate_non_negative_integer(option: str, value: Any) -> int:
     """Validate that 'value' is a positive integer or 0."""
     val = validate_integer(option, value)
     if val < 0:
-        raise ValueError("The value of %s must be " "a non negative integer" % (option,))
+        raise ValueError("The value of %s must be a non negative integer" % (option,))
     return val
 
 
-def validate_readable(option, value):
+def validate_readable(option: str, value: Any) -> Optional[str]:
     """Validates that 'value' is file-like and readable."""
     if value is None:
         return value
@@ -210,35 +224,35 @@ def validate_readable(option, value):
     return value
 
 
-def validate_positive_integer_or_none(option, value):
+def validate_positive_integer_or_none(option: str, value: Any) -> Optional[int]:
     """Validate that 'value' is a positive integer or None."""
     if value is None:
         return value
     return validate_positive_integer(option, value)
 
 
-def validate_non_negative_integer_or_none(option, value):
+def validate_non_negative_integer_or_none(option: str, value: Any) -> Optional[int]:
     """Validate that 'value' is a positive integer or 0 or None."""
     if value is None:
         return value
     return validate_non_negative_integer(option, value)
 
 
-def validate_string(option, value):
+def validate_string(option: str, value: Any) -> str:
     """Validates that 'value' is an instance of `str`."""
     if isinstance(value, str):
         return value
-    raise TypeError("Wrong type for %s, value must be an instance of " "str" % (option,))
+    raise TypeError("Wrong type for %s, value must be an instance of str" % (option,))
 
 
-def validate_string_or_none(option, value):
+def validate_string_or_none(option: str, value: Any) -> Optional[str]:
     """Validates that 'value' is an instance of `basestring` or `None`."""
     if value is None:
         return value
     return validate_string(option, value)
 
 
-def validate_int_or_basestring(option, value):
+def validate_int_or_basestring(option: str, value: Any) -> Union[int, str]:
     """Validates that 'value' is an integer or string."""
     if isinstance(value, int):
         return value
@@ -247,10 +261,10 @@ def validate_int_or_basestring(option, value):
             return int(value)
         except ValueError:
             return value
-    raise TypeError("Wrong type for %s, value must be an " "integer or a string" % (option,))
+    raise TypeError("Wrong type for %s, value must be an integer or a string" % (option,))
 
 
-def validate_non_negative_int_or_basestring(option, value):
+def validate_non_negative_int_or_basestring(option: Any, value: Any) -> Union[int, str]:
     """Validates that 'value' is an integer or string."""
     if isinstance(value, int):
         return value
@@ -261,11 +275,11 @@ def validate_non_negative_int_or_basestring(option, value):
             return value
         return validate_non_negative_integer(option, val)
     raise TypeError(
-        "Wrong type for %s, value must be an " "non negative integer or a string" % (option,)
+        "Wrong type for %s, value must be an non negative integer or a string" % (option,)
     )
 
 
-def validate_positive_float(option, value):
+def validate_positive_float(option: str, value: Any) -> float:
     """Validates that 'value' is a float, or can be converted to one, and is
     positive.
     """
@@ -280,11 +294,11 @@ def validate_positive_float(option, value):
     # float('inf') doesn't work in 2.4 or 2.5 on Windows, so just cap floats at
     # one billion - this is a reasonable approximation for infinity
     if not 0 < value < 1e9:
-        raise ValueError("%s must be greater than 0 and " "less than one billion" % (option,))
+        raise ValueError("%s must be greater than 0 and less than one billion" % (option,))
     return value
 
 
-def validate_positive_float_or_zero(option, value):
+def validate_positive_float_or_zero(option: str, value: Any) -> float:
     """Validates that 'value' is 0 or a positive float, or can be converted to
     0 or a positive float.
     """
@@ -293,7 +307,7 @@ def validate_positive_float_or_zero(option, value):
     return validate_positive_float(option, value)
 
 
-def validate_timeout_or_none(option, value):
+def validate_timeout_or_none(option: str, value: Any) -> Optional[float]:
     """Validates a timeout specified in milliseconds returning
     a value in floating point seconds.
     """
@@ -302,7 +316,7 @@ def validate_timeout_or_none(option, value):
     return validate_positive_float(option, value) / 1000.0
 
 
-def validate_timeout_or_zero(option, value):
+def validate_timeout_or_zero(option: str, value: Any) -> float:
     """Validates a timeout specified in milliseconds returning
     a value in floating point seconds for the case where None is an error
     and 0 is valid. Setting the timeout to nothing in the URI string is a
@@ -315,7 +329,7 @@ def validate_timeout_or_zero(option, value):
     return validate_positive_float(option, value) / 1000.0
 
 
-def validate_timeout_or_none_or_zero(option, value):
+def validate_timeout_or_none_or_zero(option: Any, value: Any) -> Optional[float]:
     """Validates a timeout specified in milliseconds returning
     a value in floating point seconds. value=0 and value="0" are treated the
     same as value=None which means unlimited timeout.
@@ -325,7 +339,7 @@ def validate_timeout_or_none_or_zero(option, value):
     return validate_positive_float(option, value) / 1000.0
 
 
-def validate_max_staleness(option, value):
+def validate_max_staleness(option: str, value: Any) -> int:
     """Validates maxStalenessSeconds according to the Max Staleness Spec."""
     if value == -1 or value == "-1":
         # Default: No maximum staleness.
@@ -333,14 +347,14 @@ def validate_max_staleness(option, value):
     return validate_positive_integer(option, value)
 
 
-def validate_read_preference(dummy, value):
+def validate_read_preference(dummy: Any, value: Any) -> _ServerMode:
     """Validate a read preference."""
     if not isinstance(value, _ServerMode):
         raise TypeError("%r is not a read preference." % (value,))
     return value
 
 
-def validate_read_preference_mode(dummy, value):
+def validate_read_preference_mode(dummy: Any, value: Any) -> _ServerMode:
     """Validate read preference mode for a MongoClient.
 
     .. versionchanged:: 3.5
@@ -352,14 +366,14 @@ def validate_read_preference_mode(dummy, value):
     return value
 
 
-def validate_auth_mechanism(option, value):
+def validate_auth_mechanism(option: str, value: Any) -> str:
     """Validate the authMechanism URI option."""
     if value not in MECHANISMS:
         raise ValueError("%s must be in %s" % (option, tuple(MECHANISMS)))
     return value
 
 
-def validate_uuid_representation(dummy, value):
+def validate_uuid_representation(dummy: Any, value: Any) -> int:
     """Validate the uuid representation option selected in the URI."""
     try:
         return _UUID_REPRESENTATIONS[value]
@@ -371,12 +385,12 @@ def validate_uuid_representation(dummy, value):
         )
 
 
-def validate_read_preference_tags(name, value):
+def validate_read_preference_tags(name: str, value: Any) -> List[Dict[str, str]]:
     """Parse readPreferenceTags if passed as a client kwarg."""
     if not isinstance(value, list):
         value = [value]
 
-    tag_sets = []
+    tag_sets: List = []
     for tag_set in value:
         if tag_set == "":
             tag_sets.append({})
@@ -388,7 +402,7 @@ def validate_read_preference_tags(name, value):
                 tags[unquote_plus(key)] = unquote_plus(val)
             tag_sets.append(tags)
         except Exception:
-            raise ValueError("%r not a valid " "value for %s" % (tag_set, name))
+            raise ValueError("%r not a valid value for %s" % (tag_set, name))
     return tag_sets
 
 
@@ -397,10 +411,10 @@ _MECHANISM_PROPS = frozenset(
 )
 
 
-def validate_auth_mechanism_properties(option, value):
+def validate_auth_mechanism_properties(option: str, value: Any) -> Dict[str, Union[bool, str]]:
     """Validate authMechanismProperties."""
     value = validate_string(option, value)
-    props = {}
+    props: Dict[str, Any] = {}
     for opt in value.split(","):
         try:
             key, val = opt.split(":")
@@ -430,9 +444,19 @@ def validate_auth_mechanism_properties(option, value):
     return props
 
 
-def validate_document_class(option, value):
+def validate_document_class(
+    option: str, value: Any
+) -> Union[Type[MutableMapping], Type[RawBSONDocument]]:
     """Validate the document_class option."""
-    if not issubclass(value, (abc.MutableMapping, RawBSONDocument)):
+    # issubclass can raise TypeError for generic aliases like SON[str, Any].
+    # In that case we can use the base class for the comparison.
+    is_mapping = False
+    try:
+        is_mapping = issubclass(value, abc.MutableMapping)
+    except TypeError:
+        if hasattr(value, "__origin__"):
+            is_mapping = issubclass(value.__origin__, abc.MutableMapping)
+    if not is_mapping and not issubclass(value, RawBSONDocument):
         raise TypeError(
             "%s must be dict, bson.son.SON, "
             "bson.raw_bson.RawBSONDocument, or a "
@@ -441,28 +465,28 @@ def validate_document_class(option, value):
     return value
 
 
-def validate_type_registry(option, value):
+def validate_type_registry(option: Any, value: Any) -> Optional[TypeRegistry]:
     """Validate the type_registry option."""
     if value is not None and not isinstance(value, TypeRegistry):
         raise TypeError("%s must be an instance of %s" % (option, TypeRegistry))
     return value
 
 
-def validate_list(option, value):
+def validate_list(option: str, value: Any) -> List:
     """Validates that 'value' is a list."""
     if not isinstance(value, list):
         raise TypeError("%s must be a list" % (option,))
     return value
 
 
-def validate_list_or_none(option, value):
+def validate_list_or_none(option: Any, value: Any) -> Optional[List]:
     """Validates that 'value' is a list or None."""
     if value is None:
         return value
     return validate_list(option, value)
 
 
-def validate_list_or_mapping(option, value):
+def validate_list_or_mapping(option: Any, value: Any) -> None:
     """Validates that 'value' is a list or a document."""
     if not isinstance(value, (abc.Mapping, list)):
         raise TypeError(
@@ -472,7 +496,7 @@ def validate_list_or_mapping(option, value):
         )
 
 
-def validate_is_mapping(option, value):
+def validate_is_mapping(option: str, value: Any) -> None:
     """Validate the type of method arguments that expect a document."""
     if not isinstance(value, abc.Mapping):
         raise TypeError(
@@ -482,7 +506,7 @@ def validate_is_mapping(option, value):
         )
 
 
-def validate_is_document_type(option, value):
+def validate_is_document_type(option: str, value: Any) -> None:
     """Validate the type of method arguments that expect a MongoDB document."""
     if not isinstance(value, (abc.MutableMapping, RawBSONDocument)):
         raise TypeError(
@@ -493,7 +517,7 @@ def validate_is_document_type(option, value):
         )
 
 
-def validate_appname_or_none(option, value):
+def validate_appname_or_none(option: str, value: Any) -> Optional[str]:
     """Validate the appname option."""
     if value is None:
         return value
@@ -504,7 +528,7 @@ def validate_appname_or_none(option, value):
     return value
 
 
-def validate_driver_or_none(option, value):
+def validate_driver_or_none(option: Any, value: Any) -> Optional[DriverInfo]:
     """Validate the driver keyword arg."""
     if value is None:
         return value
@@ -513,7 +537,7 @@ def validate_driver_or_none(option, value):
     return value
 
 
-def validate_server_api_or_none(option, value):
+def validate_server_api_or_none(option: Any, value: Any) -> Optional[ServerApi]:
     """Validate the server_api keyword arg."""
     if value is None:
         return value
@@ -522,7 +546,7 @@ def validate_server_api_or_none(option, value):
     return value
 
 
-def validate_is_callable_or_none(option, value):
+def validate_is_callable_or_none(option: Any, value: Any) -> Optional[Callable]:
     """Validates that 'value' is a callable."""
     if value is None:
         return value
@@ -531,7 +555,7 @@ def validate_is_callable_or_none(option, value):
     return value
 
 
-def validate_ok_for_replace(replacement):
+def validate_ok_for_replace(replacement: Mapping[str, Any]) -> None:
     """Validate a replacement document."""
     validate_is_mapping("replacement", replacement)
     # Replacement can be {}
@@ -541,7 +565,7 @@ def validate_ok_for_replace(replacement):
             raise ValueError("replacement can not include $ operators")
 
 
-def validate_ok_for_update(update):
+def validate_ok_for_update(update: Any) -> None:
     """Validate an update document."""
     validate_list_or_mapping("update", update)
     # Update cannot be {}.
@@ -557,7 +581,7 @@ def validate_ok_for_update(update):
 _UNICODE_DECODE_ERROR_HANDLERS = frozenset(["strict", "replace", "ignore"])
 
 
-def validate_unicode_decode_error_handler(dummy, value):
+def validate_unicode_decode_error_handler(dummy: Any, value: str) -> str:
     """Validate the Unicode decode error handler option of CodecOptions."""
     if value not in _UNICODE_DECODE_ERROR_HANDLERS:
         raise ValueError(
@@ -568,14 +592,14 @@ def validate_unicode_decode_error_handler(dummy, value):
     return value
 
 
-def validate_tzinfo(dummy, value):
+def validate_tzinfo(dummy: Any, value: Any) -> Optional[datetime.tzinfo]:
     """Validate the tzinfo option"""
     if value is not None and not isinstance(value, datetime.tzinfo):
         raise TypeError("%s must be an instance of datetime.tzinfo" % value)
     return value
 
 
-def validate_auto_encryption_opts_or_none(option, value):
+def validate_auto_encryption_opts_or_none(option: Any, value: Any) -> Optional[Any]:
     """Validate the driver keyword arg."""
     if value is None:
         return value
@@ -589,7 +613,7 @@ def validate_auto_encryption_opts_or_none(option, value):
 
 # Dictionary where keys are the names of public URI options, and values
 # are lists of aliases for that option.
-URI_OPTIONS_ALIAS_MAP = {
+URI_OPTIONS_ALIAS_MAP: Dict[str, List[str]] = {
     "tls": ["ssl"],
 }
 
@@ -597,7 +621,7 @@ URI_OPTIONS_ALIAS_MAP = {
 # are functions that validate user-input values for that option. If an option
 # alias uses a different validator than its public counterpart, it should be
 # included here as a key, value pair.
-URI_OPTIONS_VALIDATOR_MAP = {
+URI_OPTIONS_VALIDATOR_MAP: Dict[str, Callable[[Any, Any], Any]] = {
     "appname": validate_appname_or_none,
     "authmechanism": validate_auth_mechanism,
     "authmechanismproperties": validate_auth_mechanism_properties,
@@ -638,7 +662,7 @@ URI_OPTIONS_VALIDATOR_MAP = {
 
 # Dictionary where keys are the names of URI options specific to pymongo,
 # and values are functions that validate user-input values for those options.
-NONSPEC_OPTIONS_VALIDATOR_MAP = {
+NONSPEC_OPTIONS_VALIDATOR_MAP: Dict[str, Callable[[Any, Any], Any]] = {
     "connect": validate_boolean_or_string,
     "driver": validate_driver_or_none,
     "server_api": validate_server_api_or_none,
@@ -655,7 +679,7 @@ NONSPEC_OPTIONS_VALIDATOR_MAP = {
 # Dictionary where keys are the names of keyword-only options for the
 # MongoClient constructor, and values are functions that validate user-input
 # values for those options.
-KW_VALIDATORS = {
+KW_VALIDATORS: Dict[str, Callable[[Any, Any], Any]] = {
     "document_class": validate_document_class,
     "type_registry": validate_type_registry,
     "read_preference": validate_read_preference,
@@ -671,14 +695,14 @@ KW_VALIDATORS = {
 # internally-used names of that URI option. Options with only one name
 # variant need not be included here. Options whose public and internal
 # names are the same need not be included here.
-INTERNAL_URI_OPTION_NAME_MAP = {
+INTERNAL_URI_OPTION_NAME_MAP: Dict[str, str] = {
     "ssl": "tls",
 }
 
 # Map from deprecated URI option names to a tuple indicating the method of
 # their deprecation and any additional information that may be needed to
 # construct the warning message.
-URI_OPTIONS_DEPRECATION_MAP = {
+URI_OPTIONS_DEPRECATION_MAP: Dict[str, Tuple[str, str]] = {
     # format: <deprecated option name>: (<mode>, <message>),
     # Supported <mode> values:
     # - 'renamed': <message> should be the new option name. Note that case is
@@ -697,11 +721,11 @@ for optname, aliases in URI_OPTIONS_ALIAS_MAP.items():
             URI_OPTIONS_VALIDATOR_MAP[alias] = URI_OPTIONS_VALIDATOR_MAP[optname]
 
 # Map containing all URI option and keyword argument validators.
-VALIDATORS = URI_OPTIONS_VALIDATOR_MAP.copy()
+VALIDATORS: Dict[str, Callable[[Any, Any], Any]] = URI_OPTIONS_VALIDATOR_MAP.copy()
 VALIDATORS.update(KW_VALIDATORS)
 
 # List of timeout-related options.
-TIMEOUT_OPTIONS = [
+TIMEOUT_OPTIONS: List[str] = [
     "connecttimeoutms",
     "heartbeatfrequencyms",
     "maxidletimems",
@@ -715,15 +739,15 @@ TIMEOUT_OPTIONS = [
 _AUTH_OPTIONS = frozenset(["authmechanismproperties"])
 
 
-def validate_auth_option(option, value):
+def validate_auth_option(option: str, value: Any) -> Tuple[str, Any]:
     """Validate optional authentication parameters."""
     lower, value = validate(option, value)
     if lower not in _AUTH_OPTIONS:
-        raise ConfigurationError("Unknown " "authentication option: %s" % (option,))
+        raise ConfigurationError("Unknown authentication option: %s" % (option,))
     return option, value
 
 
-def validate(option, value):
+def validate(option: str, value: Any) -> Tuple[str, Any]:
     """Generic validation function."""
     lower = option.lower()
     validator = VALIDATORS.get(lower, raise_config_error)
@@ -731,7 +755,9 @@ def validate(option, value):
     return option, value
 
 
-def get_validated_options(options, warn=True):
+def get_validated_options(
+    options: Mapping[str, Any], warn: bool = True
+) -> MutableMapping[str, Any]:
     """Validate each entry in options and raise a warning if it is not valid.
     Returns a copy of options with invalid entries removed.
 
@@ -741,14 +767,15 @@ def get_validated_options(options, warn=True):
           invalid options will be ignored. Otherwise, invalid options will
           cause errors.
     """
+    validated_options: MutableMapping[str, Any]
     if isinstance(options, _CaseInsensitiveDictionary):
         validated_options = _CaseInsensitiveDictionary()
-        get_normed_key = lambda x: x
-        get_setter_key = lambda x: options.cased_key(x)
+        get_normed_key = lambda x: x  # noqa: E731
+        get_setter_key = lambda x: options.cased_key(x)  # noqa: E731
     else:
         validated_options = {}
-        get_normed_key = lambda x: x.lower()
-        get_setter_key = lambda x: x
+        get_normed_key = lambda x: x.lower()  # noqa: E731
+        get_setter_key = lambda x: x  # noqa: E731
 
     for opt, value in options.items():
         normed_key = get_normed_key(opt)
@@ -776,12 +803,16 @@ class BaseObject(object):
     SHOULD NOT BE USED BY DEVELOPERS EXTERNAL TO MONGODB.
     """
 
-    def __init__(self, codec_options, read_preference, write_concern, read_concern):
+    def __init__(
+        self,
+        codec_options: CodecOptions,
+        read_preference: _ServerMode,
+        write_concern: WriteConcern,
+        read_concern: ReadConcern,
+    ) -> None:
 
         if not isinstance(codec_options, CodecOptions):
-            raise TypeError(
-                "codec_options must be an instance of " "bson.codec_options.CodecOptions"
-            )
+            raise TypeError("codec_options must be an instance of bson.codec_options.CodecOptions")
         self.__codec_options = codec_options
 
         if not isinstance(read_preference, _ServerMode):
@@ -794,25 +825,23 @@ class BaseObject(object):
 
         if not isinstance(write_concern, WriteConcern):
             raise TypeError(
-                "write_concern must be an instance of " "pymongo.write_concern.WriteConcern"
+                "write_concern must be an instance of pymongo.write_concern.WriteConcern"
             )
         self.__write_concern = write_concern
 
         if not isinstance(read_concern, ReadConcern):
-            raise TypeError(
-                "read_concern must be an instance of " "pymongo.read_concern.ReadConcern"
-            )
+            raise TypeError("read_concern must be an instance of pymongo.read_concern.ReadConcern")
         self.__read_concern = read_concern
 
     @property
-    def codec_options(self):
+    def codec_options(self) -> CodecOptions:
         """Read only access to the :class:`~bson.codec_options.CodecOptions`
         of this instance.
         """
         return self.__codec_options
 
     @property
-    def write_concern(self):
+    def write_concern(self) -> WriteConcern:
         """Read only access to the :class:`~pymongo.write_concern.WriteConcern`
         of this instance.
 
@@ -829,7 +858,7 @@ class BaseObject(object):
         return self.write_concern
 
     @property
-    def read_preference(self):
+    def read_preference(self) -> _ServerMode:
         """Read only access to the read preference of this instance.
 
         .. versionchanged:: 3.0
@@ -845,7 +874,7 @@ class BaseObject(object):
         return self.__read_preference
 
     @property
-    def read_concern(self):
+    def read_concern(self) -> ReadConcern:
         """Read only access to the :class:`~pymongo.read_concern.ReadConcern`
         of this instance.
 
