@@ -15,14 +15,13 @@
 """Support for explicit client-side field level encryption."""
 
 import contextlib
-import os
-import subprocess
 import uuid
 import weakref
+from typing import Any, Mapping, Optional, Sequence
 
 try:
     from pymongocrypt.auto_encrypter import AutoEncrypter
-    from pymongocrypt.errors import MongoCryptError
+    from pymongocrypt.errors import MongoCryptError  # noqa: F401
     from pymongocrypt.explicit_encrypter import ExplicitEncrypter
     from pymongocrypt.mongocrypt import MongoCryptOptions
     from pymongocrypt.state_machine import MongoCryptCallback
@@ -39,6 +38,7 @@ from bson.errors import BSONError
 from bson.raw_bson import DEFAULT_RAW_BSON_OPTIONS, RawBSONDocument, _inflate_bson
 from bson.son import SON
 from pymongo.daemon import _spawn_daemon
+from pymongo.encryption_options import AutoEncryptionOpts
 from pymongo.errors import (
     ConfigurationError,
     EncryptionError,
@@ -48,20 +48,15 @@ from pymongo.errors import (
 from pymongo.mongo_client import MongoClient
 from pymongo.pool import PoolOptions, _configured_socket
 from pymongo.read_concern import ReadConcern
-from pymongo.ssl_support import HAVE_SSL, get_ssl_context
+from pymongo.ssl_support import get_ssl_context
 from pymongo.uri_parser import parse_host
 from pymongo.write_concern import WriteConcern
 
-if HAVE_SSL:
-    from ssl import CERT_REQUIRED
-else:
-    CERT_REQUIRED = None
-
 _HTTPS_PORT = 443
 _KMS_CONNECT_TIMEOUT = 10  # TODO: CDRIVER-3262 will define this value.
 _MONGOCRYPTD_TIMEOUT_MS = 10000
 
-_DATA_KEY_OPTS = CodecOptions(document_class=SON, uuid_representation=STANDARD)
+_DATA_KEY_OPTS: CodecOptions = CodecOptions(document_class=SON, uuid_representation=STANDARD)
 # Use RawBSONDocument codec options to avoid needlessly decoding
 # documents from the key vault.
 _KEY_VAULT_OPTS = CodecOptions(document_class=RawBSONDocument, uuid_representation=STANDARD)
@@ -80,9 +75,10 @@ def _wrap_encryption_errors():
         raise EncryptionError(exc)
 
 
-class _EncryptionIO(MongoCryptCallback):
+class _EncryptionIO(MongoCryptCallback):  # type: ignore
     def __init__(self, client, key_vault_coll, mongocryptd_client, opts):
         """Internal class to perform I/O on behalf of pymongocrypt."""
+        self.client_ref: Any
         # Use a weak ref to break reference cycle.
         if client is not None:
             self.client_ref = weakref.ref(client)
@@ -108,24 +104,26 @@ class _EncryptionIO(MongoCryptCallback):
         """
         endpoint = kms_context.endpoint
         message = kms_context.message
-        host, port = parse_host(endpoint, _HTTPS_PORT)
-        # Enable strict certificate verification, OCSP, match hostname, and
-        # SNI using the system default CA certificates.
-        ctx = get_ssl_context(
-            None,  # certfile
-            None,  # keyfile
-            None,  # passphrase
-            None,  # ca_certs
-            CERT_REQUIRED,  # cert_reqs
-            None,  # crlfile
-            True,  # match_hostname
-            True,
-        )  # check_ocsp_endpoint
+        provider = kms_context.kms_provider
+        ctx = self.opts._kms_ssl_contexts.get(provider)
+        if ctx is None:
+            # Enable strict certificate verification, OCSP, match hostname, and
+            # SNI using the system default CA certificates.
+            ctx = get_ssl_context(
+                None,  # certfile
+                None,  # passphrase
+                None,  # ca_certs
+                None,  # crlfile
+                False,  # allow_invalid_certificates
+                False,  # allow_invalid_hostnames
+                False,
+            )  # disable_ocsp_endpoint_check
         opts = PoolOptions(
             connect_timeout=_KMS_CONNECT_TIMEOUT,
             socket_timeout=_KMS_CONNECT_TIMEOUT,
             ssl_context=ctx,
         )
+        host, port = parse_host(endpoint, _HTTPS_PORT)
         conn = _configured_socket((host, port), opts)
         try:
             conn.sendall(message)
@@ -215,7 +213,7 @@ class _EncryptionIO(MongoCryptCallback):
         :Returns:
           The _id of the inserted data key document.
         """
-        raw_doc = RawBSONDocument(data_key)
+        raw_doc = RawBSONDocument(data_key, _KEY_VAULT_OPTS)
         data_key_id = raw_doc.get("_id")
         if not isinstance(data_key_id, uuid.UUID):
             raise TypeError("data_key _id must be a UUID")
@@ -292,7 +290,7 @@ class _Encrypter(object):
         db, coll = opts._key_vault_namespace.split(".", 1)
         key_vault_coll = key_vault_client[db][coll]
 
-        mongocryptd_client = MongoClient(
+        mongocryptd_client: MongoClient = MongoClient(
             opts._mongocryptd_uri, connect=False, serverSelectionTimeoutMS=_MONGOCRYPTD_TIMEOUT_MS
         )
 
@@ -302,29 +300,23 @@ class _Encrypter(object):
         )
         self._closed = False
 
-    def encrypt(self, database, cmd, check_keys, codec_options):
+    def encrypt(self, database, cmd, codec_options):
         """Encrypt a MongoDB command.
 
         :Parameters:
           - `database`: The database for this command.
           - `cmd`: A command document.
-          - `check_keys`: If True, check `cmd` for invalid keys.
           - `codec_options`: The CodecOptions to use while encoding `cmd`.
 
         :Returns:
           The encrypted command to execute.
         """
         self._check_closed()
-        # Workaround for $clusterTime which is incompatible with
-        # check_keys.
-        cluster_time = check_keys and cmd.pop("$clusterTime", None)
-        encoded_cmd = _dict_to_bson(cmd, check_keys, codec_options)
+        encoded_cmd = _dict_to_bson(cmd, False, codec_options)
         with _wrap_encryption_errors():
             encrypted_cmd = self._auto_encrypter.encrypt(database, encoded_cmd)
             # TODO: PYTHON-1922 avoid decoding the encrypted_cmd.
             encrypt_cmd = _inflate_bson(encrypted_cmd, DEFAULT_RAW_BSON_OPTIONS)
-            if cluster_time:
-                encrypt_cmd["$clusterTime"] = cluster_time
             return encrypt_cmd
 
     def decrypt(self, response):
@@ -363,7 +355,14 @@ class Algorithm(object):
 class ClientEncryption(object):
     """Explicit client-side field level encryption."""
 
-    def __init__(self, kms_providers, key_vault_namespace, key_vault_client, codec_options):
+    def __init__(
+        self,
+        kms_providers: Mapping[str, Any],
+        key_vault_namespace: str,
+        key_vault_client: MongoClient,
+        codec_options: CodecOptions,
+        kms_tls_options: Optional[Mapping[str, Any]] = None,
+    ) -> None:
         """Explicit client-side field level encryption.
 
         The ClientEncryption class encapsulates explicit operations on a key
@@ -378,9 +377,8 @@ class ClientEncryption(object):
         See :ref:`explicit-client-side-encryption` for an example.
 
         :Parameters:
-          - `kms_providers`: Map of KMS provider options. Two KMS providers
-            are supported: "aws" and "local". The kmsProviders map values
-            differ by provider:
+          - `kms_providers`: Map of KMS provider options. The `kms_providers`
+            map values differ by provider:
 
               - `aws`: Map with "accessKeyId" and "secretAccessKey" as strings.
                 These are the AWS access key ID and AWS secret access key used
@@ -392,12 +390,14 @@ class ClientEncryption(object):
                 These are the Azure Active Directory credentials used to
                 generate Azure Key Vault messages.
               - `gcp`: Map with "email" as a string and "privateKey"
-                as `bytes` or a base64 encoded string (unicode on Python 2).
+                as `bytes` or a base64 encoded string.
                 Additionally, "endpoint" may also be specified as a string
                 (defaults to 'oauth2.googleapis.com'). These are the
                 credentials used to generate Google Cloud KMS messages.
+              - `kmip`: Map with "endpoint" as a host with required port.
+                For example: ``{"endpoint": "example.com:443"}``.
               - `local`: Map with "key" as `bytes` (96 bytes in length) or
-                a base64 encoded string (unicode on Python 2) which decodes
+                a base64 encoded string which decodes
                 to 96 bytes. "key" is the master key used to encrypt/decrypt
                 data keys. This key should be generated and stored as securely
                 as possible.
@@ -415,6 +415,20 @@ class ClientEncryption(object):
             should be the same CodecOptions instance configured on the
             MongoClient, Database, or Collection used to access application
             data.
+          - `kms_tls_options` (optional): A map of KMS provider names to TLS
+            options to use when creating secure connections to KMS providers.
+            Accepts the same TLS options as
+            :class:`pymongo.mongo_client.MongoClient`. For example, to
+            override the system default CA file::
+
+              kms_tls_options={'kmip': {'tlsCAFile': certifi.where()}}
+
+            Or to supply a client certificate::
+
+              kms_tls_options={'kmip': {'tlsCertificateKeyFile': 'client.pem'}}
+
+        .. versionchanged:: 4.0
+           Added the `kms_tls_options` parameter and the "kmip" KMS provider.
 
         .. versionadded:: 3.9
         """
@@ -426,9 +440,7 @@ class ClientEncryption(object):
             )
 
         if not isinstance(codec_options, CodecOptions):
-            raise TypeError(
-                "codec_options must be an instance of " "bson.codec_options.CodecOptions"
-            )
+            raise TypeError("codec_options must be an instance of bson.codec_options.CodecOptions")
 
         self._kms_providers = kms_providers
         self._key_vault_namespace = key_vault_namespace
@@ -438,17 +450,27 @@ class ClientEncryption(object):
         db, coll = key_vault_namespace.split(".", 1)
         key_vault_coll = key_vault_client[db][coll]
 
-        self._io_callbacks = _EncryptionIO(None, key_vault_coll, None, None)
+        opts = AutoEncryptionOpts(
+            kms_providers, key_vault_namespace, kms_tls_options=kms_tls_options
+        )
+        self._io_callbacks: Optional[_EncryptionIO] = _EncryptionIO(
+            None, key_vault_coll, None, opts
+        )
         self._encryption = ExplicitEncrypter(
             self._io_callbacks, MongoCryptOptions(kms_providers, None)
         )
 
-    def create_data_key(self, kms_provider, master_key=None, key_alt_names=None):
+    def create_data_key(
+        self,
+        kms_provider: str,
+        master_key: Optional[Mapping[str, Any]] = None,
+        key_alt_names: Optional[Sequence[str]] = None,
+    ) -> Binary:
         """Create and insert a new data key into the key vault collection.
 
         :Parameters:
           - `kms_provider`: The KMS provider to use. Supported values are
-            "aws" and "local".
+            "aws", "azure", "gcp", "kmip", and "local".
           - `master_key`: Identifies a KMS-specific key used to encrypt the
             new data key. If the kmsProvider is "local" the `master_key` is
             not applicable and may be omitted.
@@ -483,6 +505,16 @@ class ClientEncryption(object):
               - `endpoint` (string): Optional. Host with optional port.
                 Defaults to "cloudkms.googleapis.com".
 
+            If the `kms_provider` is "kmip" it is optional and has the
+            following fields::
+
+              - `keyId` (string): Optional. `keyId` is the KMIP Unique
+                Identifier to a 96 byte KMIP Secret Data managed object. If
+                keyId is omitted, the driver creates a random 96 byte KMIP
+                Secret Data managed object.
+              - `endpoint` (string): Optional. Host with optional
+                 port, e.g. "example.vault.azure.net:".
+
           - `key_alt_names` (optional): An optional list of string alternate
             names used to reference a key. If a key is created with alternate
             names, then encryption may refer to the key by the unique alternate
@@ -505,7 +537,13 @@ class ClientEncryption(object):
                 kms_provider, master_key=master_key, key_alt_names=key_alt_names
             )
 
-    def encrypt(self, value, algorithm, key_id=None, key_alt_name=None):
+    def encrypt(
+        self,
+        value: Any,
+        algorithm: str,
+        key_id: Optional[Binary] = None,
+        key_alt_name: Optional[str] = None,
+    ) -> Binary:
         """Encrypt a BSON value with a given key and algorithm.
 
         Note that exactly one of ``key_id`` or  ``key_alt_name`` must be
@@ -534,9 +572,9 @@ class ClientEncryption(object):
             encrypted_doc = self._encryption.encrypt(
                 doc, algorithm, key_id=key_id, key_alt_name=key_alt_name
             )
-            return decode(encrypted_doc)["v"]
+            return decode(encrypted_doc)["v"]  # type: ignore[index]
 
-    def decrypt(self, value):
+    def decrypt(self, value: Binary) -> Any:
         """Decrypt an encrypted value.
 
         :Parameters:
@@ -555,17 +593,17 @@ class ClientEncryption(object):
             decrypted_doc = self._encryption.decrypt(doc)
             return decode(decrypted_doc, codec_options=self._codec_options)["v"]
 
-    def __enter__(self):
+    def __enter__(self) -> "ClientEncryption":
         return self
 
-    def __exit__(self, exc_type, exc_val, exc_tb):
+    def __exit__(self, exc_type: Any, exc_val: Any, exc_tb: Any) -> None:
         self.close()
 
     def _check_closed(self):
         if self._encryption is None:
             raise InvalidOperation("Cannot use closed ClientEncryption")
 
-    def close(self):
+    def close(self) -> None:
         """Release resources.
 
         Note that using this class in a with-statement will automatically call
