@@ -180,13 +180,22 @@ will not add that listener to existing client instances.
   handler first.
 """
 
+import datetime
 from collections import abc, namedtuple
+from typing import TYPE_CHECKING, Any, Dict, Optional
 
-from pymongo.hello import HelloCompat
+from bson.objectid import ObjectId
+from pymongo.hello import Hello, HelloCompat
 from pymongo.helpers import _handle_exception
+from pymongo.typings import _Address, _DocumentOut
+
+if TYPE_CHECKING:
+    from pymongo.server_description import ServerDescription
+    from pymongo.topology_description import TopologyDescription
+
 
 _Listeners = namedtuple(
-    "Listeners",
+    "_Listeners",
     (
         "command_listeners",
         "server_listeners",
@@ -210,7 +219,7 @@ class CommandListener(_EventListener):
     and `CommandFailedEvent`.
     """
 
-    def started(self, event):
+    def started(self, event: "CommandStartedEvent") -> None:
         """Abstract method to handle a `CommandStartedEvent`.
 
         :Parameters:
@@ -218,7 +227,7 @@ class CommandListener(_EventListener):
         """
         raise NotImplementedError
 
-    def succeeded(self, event):
+    def succeeded(self, event: "CommandSucceededEvent") -> None:
         """Abstract method to handle a `CommandSucceededEvent`.
 
         :Parameters:
@@ -226,7 +235,7 @@ class CommandListener(_EventListener):
         """
         raise NotImplementedError
 
-    def failed(self, event):
+    def failed(self, event: "CommandFailedEvent") -> None:
         """Abstract method to handle a `CommandFailedEvent`.
 
         :Parameters:
@@ -251,7 +260,7 @@ class ConnectionPoolListener(_EventListener):
     .. versionadded:: 3.9
     """
 
-    def pool_created(self, event):
+    def pool_created(self, event: "PoolCreatedEvent") -> None:
         """Abstract method to handle a :class:`PoolCreatedEvent`.
 
         Emitted when a Connection Pool is created.
@@ -261,7 +270,7 @@ class ConnectionPoolListener(_EventListener):
         """
         raise NotImplementedError
 
-    def pool_ready(self, event):
+    def pool_ready(self, event: "PoolReadyEvent") -> None:
         """Abstract method to handle a :class:`PoolReadyEvent`.
 
         Emitted when a Connection Pool is marked ready.
@@ -273,7 +282,7 @@ class ConnectionPoolListener(_EventListener):
         """
         raise NotImplementedError
 
-    def pool_cleared(self, event):
+    def pool_cleared(self, event: "PoolClearedEvent") -> None:
         """Abstract method to handle a `PoolClearedEvent`.
 
         Emitted when a Connection Pool is cleared.
@@ -283,7 +292,7 @@ class ConnectionPoolListener(_EventListener):
         """
         raise NotImplementedError
 
-    def pool_closed(self, event):
+    def pool_closed(self, event: "PoolClosedEvent") -> None:
         """Abstract method to handle a `PoolClosedEvent`.
 
         Emitted when a Connection Pool is closed.
@@ -293,7 +302,7 @@ class ConnectionPoolListener(_EventListener):
         """
         raise NotImplementedError
 
-    def connection_created(self, event):
+    def connection_created(self, event: "ConnectionCreatedEvent") -> None:
         """Abstract method to handle a :class:`ConnectionCreatedEvent`.
 
         Emitted when a Connection Pool creates a Connection object.
@@ -303,7 +312,7 @@ class ConnectionPoolListener(_EventListener):
         """
         raise NotImplementedError
 
-    def connection_ready(self, event):
+    def connection_ready(self, event: "ConnectionReadyEvent") -> None:
         """Abstract method to handle a :class:`ConnectionReadyEvent`.
 
         Emitted when a Connection has finished its setup, and is now ready to
@@ -314,7 +323,7 @@ class ConnectionPoolListener(_EventListener):
         """
         raise NotImplementedError
 
-    def connection_closed(self, event):
+    def connection_closed(self, event: "ConnectionClosedEvent") -> None:
         """Abstract method to handle a :class:`ConnectionClosedEvent`.
 
         Emitted when a Connection Pool closes a Connection.
@@ -324,7 +333,7 @@ class ConnectionPoolListener(_EventListener):
         """
         raise NotImplementedError
 
-    def connection_check_out_started(self, event):
+    def connection_check_out_started(self, event: "ConnectionCheckOutStartedEvent") -> None:
         """Abstract method to handle a :class:`ConnectionCheckOutStartedEvent`.
 
         Emitted when the driver starts attempting to check out a connection.
@@ -334,7 +343,7 @@ class ConnectionPoolListener(_EventListener):
         """
         raise NotImplementedError
 
-    def connection_check_out_failed(self, event):
+    def connection_check_out_failed(self, event: "ConnectionCheckOutFailedEvent") -> None:
         """Abstract method to handle a :class:`ConnectionCheckOutFailedEvent`.
 
         Emitted when the driver's attempt to check out a connection fails.
@@ -344,7 +353,7 @@ class ConnectionPoolListener(_EventListener):
         """
         raise NotImplementedError
 
-    def connection_checked_out(self, event):
+    def connection_checked_out(self, event: "ConnectionCheckedOutEvent") -> None:
         """Abstract method to handle a :class:`ConnectionCheckedOutEvent`.
 
         Emitted when the driver successfully checks out a Connection.
@@ -354,7 +363,7 @@ class ConnectionPoolListener(_EventListener):
         """
         raise NotImplementedError
 
-    def connection_checked_in(self, event):
+    def connection_checked_in(self, event: "ConnectionCheckedInEvent") -> None:
         """Abstract method to handle a :class:`ConnectionCheckedInEvent`.
 
         Emitted when the driver checks in a Connection back to the Connection
@@ -375,7 +384,7 @@ class ServerHeartbeatListener(_EventListener):
     .. versionadded:: 3.3
     """
 
-    def started(self, event):
+    def started(self, event: "ServerHeartbeatStartedEvent") -> None:
         """Abstract method to handle a `ServerHeartbeatStartedEvent`.
 
         :Parameters:
@@ -383,7 +392,7 @@ class ServerHeartbeatListener(_EventListener):
         """
         raise NotImplementedError
 
-    def succeeded(self, event):
+    def succeeded(self, event: "ServerHeartbeatSucceededEvent") -> None:
         """Abstract method to handle a `ServerHeartbeatSucceededEvent`.
 
         :Parameters:
@@ -391,7 +400,7 @@ class ServerHeartbeatListener(_EventListener):
         """
         raise NotImplementedError
 
-    def failed(self, event):
+    def failed(self, event: "ServerHeartbeatFailedEvent") -> None:
         """Abstract method to handle a `ServerHeartbeatFailedEvent`.
 
         :Parameters:
@@ -408,7 +417,7 @@ class TopologyListener(_EventListener):
     .. versionadded:: 3.3
     """
 
-    def opened(self, event):
+    def opened(self, event: "TopologyOpenedEvent") -> None:
         """Abstract method to handle a `TopologyOpenedEvent`.
 
         :Parameters:
@@ -416,7 +425,7 @@ class TopologyListener(_EventListener):
         """
         raise NotImplementedError
 
-    def description_changed(self, event):
+    def description_changed(self, event: "TopologyDescriptionChangedEvent") -> None:
         """Abstract method to handle a `TopologyDescriptionChangedEvent`.
 
         :Parameters:
@@ -424,7 +433,7 @@ class TopologyListener(_EventListener):
         """
         raise NotImplementedError
 
-    def closed(self, event):
+    def closed(self, event: "TopologyClosedEvent") -> None:
         """Abstract method to handle a `TopologyClosedEvent`.
 
         :Parameters:
@@ -441,7 +450,7 @@ class ServerListener(_EventListener):
     .. versionadded:: 3.3
     """
 
-    def opened(self, event):
+    def opened(self, event: "ServerOpeningEvent") -> None:
         """Abstract method to handle a `ServerOpeningEvent`.
 
         :Parameters:
@@ -449,7 +458,7 @@ class ServerListener(_EventListener):
         """
         raise NotImplementedError
 
-    def description_changed(self, event):
+    def description_changed(self, event: "ServerDescriptionChangedEvent") -> None:
         """Abstract method to handle a `ServerDescriptionChangedEvent`.
 
         :Parameters:
@@ -457,7 +466,7 @@ class ServerListener(_EventListener):
         """
         raise NotImplementedError
 
-    def closed(self, event):
+    def closed(self, event: "ServerClosedEvent") -> None:
         """Abstract method to handle a `ServerClosedEvent`.
 
         :Parameters:
@@ -486,7 +495,7 @@ def _validate_event_listeners(option, listeners):
     return listeners
 
 
-def register(listener):
+def register(listener: _EventListener) -> None:
     """Register a global event listener.
 
     :Parameters:
@@ -547,7 +556,14 @@ class _CommandEvent(object):
 
     __slots__ = ("__cmd_name", "__rqst_id", "__conn_id", "__op_id", "__service_id")
 
-    def __init__(self, command_name, request_id, connection_id, operation_id, service_id=None):
+    def __init__(
+        self,
+        command_name: str,
+        request_id: int,
+        connection_id: _Address,
+        operation_id: Optional[int],
+        service_id: Optional[ObjectId] = None,
+    ) -> None:
         self.__cmd_name = command_name
         self.__rqst_id = request_id
         self.__conn_id = connection_id
@@ -555,22 +571,22 @@ class _CommandEvent(object):
         self.__service_id = service_id
 
     @property
-    def command_name(self):
+    def command_name(self) -> str:
         """The command name."""
         return self.__cmd_name
 
     @property
-    def request_id(self):
+    def request_id(self) -> int:
         """The request id for this operation."""
         return self.__rqst_id
 
     @property
-    def connection_id(self):
+    def connection_id(self) -> _Address:
         """The address (host, port) of the server this command was sent to."""
         return self.__conn_id
 
     @property
-    def service_id(self):
+    def service_id(self) -> Optional[ObjectId]:
         """The service_id this command was sent to, or ``None``.
 
         .. versionadded:: 3.12
@@ -578,7 +594,7 @@ class _CommandEvent(object):
         return self.__service_id
 
     @property
-    def operation_id(self):
+    def operation_id(self) -> Optional[int]:
         """An id for this series of events or None."""
         return self.__op_id
 
@@ -598,31 +614,41 @@ class CommandStartedEvent(_CommandEvent):
 
     __slots__ = ("__cmd", "__db")
 
-    def __init__(self, command, database_name, *args, service_id=None):
+    def __init__(
+        self,
+        command: _DocumentOut,
+        database_name: str,
+        request_id: int,
+        connection_id: _Address,
+        operation_id: Optional[int],
+        service_id: Optional[ObjectId] = None,
+    ) -> None:
         if not command:
             raise ValueError("%r is not a valid command" % (command,))
         # Command name must be first key.
         command_name = next(iter(command))
-        super(CommandStartedEvent, self).__init__(command_name, *args, service_id=service_id)
-        cmd_name, cmd_doc = command_name.lower(), command[command_name]
+        super(CommandStartedEvent, self).__init__(
+            command_name, request_id, connection_id, operation_id, service_id=service_id
+        )
+        cmd_name = command_name.lower()
         if cmd_name in _SENSITIVE_COMMANDS or _is_speculative_authenticate(cmd_name, command):
-            self.__cmd = {}
+            self.__cmd: _DocumentOut = {}
         else:
             self.__cmd = command
         self.__db = database_name
 
     @property
-    def command(self):
+    def command(self) -> _DocumentOut:
         """The command document."""
         return self.__cmd
 
     @property
-    def database_name(self):
+    def database_name(self) -> str:
         """The name of the database this command was run against."""
         return self.__db
 
     def __repr__(self):
-        return ("<%s %s db: %r, command: %r, operation_id: %s, " "service_id: %s>") % (
+        return ("<%s %s db: %r, command: %r, operation_id: %s, service_id: %s>") % (
             self.__class__.__name__,
             self.connection_id,
             self.database_name,
@@ -650,36 +676,36 @@ class CommandSucceededEvent(_CommandEvent):
 
     def __init__(
         self,
-        duration,
-        reply,
-        command_name,
-        request_id,
-        connection_id,
-        operation_id,
-        service_id=None,
-    ):
+        duration: datetime.timedelta,
+        reply: _DocumentOut,
+        command_name: str,
+        request_id: int,
+        connection_id: _Address,
+        operation_id: Optional[int],
+        service_id: Optional[ObjectId] = None,
+    ) -> None:
         super(CommandSucceededEvent, self).__init__(
             command_name, request_id, connection_id, operation_id, service_id=service_id
         )
         self.__duration_micros = _to_micros(duration)
         cmd_name = command_name.lower()
         if cmd_name in _SENSITIVE_COMMANDS or _is_speculative_authenticate(cmd_name, reply):
-            self.__reply = {}
+            self.__reply: _DocumentOut = {}
         else:
             self.__reply = reply
 
     @property
-    def duration_micros(self):
+    def duration_micros(self) -> int:
         """The duration of this operation in microseconds."""
         return self.__duration_micros
 
     @property
-    def reply(self):
+    def reply(self) -> _DocumentOut:
         """The server failure document for this operation."""
         return self.__reply
 
     def __repr__(self):
-        return ("<%s %s command: %r, operation_id: %s, duration_micros: %s, " "service_id: %s>") % (
+        return ("<%s %s command: %r, operation_id: %s, duration_micros: %s, service_id: %s>") % (
             self.__class__.__name__,
             self.connection_id,
             self.command_name,
@@ -705,18 +731,29 @@ class CommandFailedEvent(_CommandEvent):
 
     __slots__ = ("__duration_micros", "__failure")
 
-    def __init__(self, duration, failure, *args, service_id=None):
-        super(CommandFailedEvent, self).__init__(*args, service_id=service_id)
+    def __init__(
+        self,
+        duration: datetime.timedelta,
+        failure: _DocumentOut,
+        command_name: str,
+        request_id: int,
+        connection_id: _Address,
+        operation_id: Optional[int],
+        service_id: Optional[ObjectId] = None,
+    ) -> None:
+        super(CommandFailedEvent, self).__init__(
+            command_name, request_id, connection_id, operation_id, service_id=service_id
+        )
         self.__duration_micros = _to_micros(duration)
         self.__failure = failure
 
     @property
-    def duration_micros(self):
+    def duration_micros(self) -> int:
         """The duration of this operation in microseconds."""
         return self.__duration_micros
 
     @property
-    def failure(self):
+    def failure(self) -> _DocumentOut:
         """The server failure document for this operation."""
         return self.__failure
 
@@ -740,11 +777,11 @@ class _PoolEvent(object):
 
     __slots__ = ("__address",)
 
-    def __init__(self, address):
+    def __init__(self, address: _Address) -> None:
         self.__address = address
 
     @property
-    def address(self):
+    def address(self) -> _Address:
         """The address (host, port) pair of the server the pool is attempting
         to connect to.
         """
@@ -766,12 +803,12 @@ class PoolCreatedEvent(_PoolEvent):
 
     __slots__ = ("__options",)
 
-    def __init__(self, address, options):
+    def __init__(self, address: _Address, options: Dict[str, Any]) -> None:
         super(PoolCreatedEvent, self).__init__(address)
         self.__options = options
 
     @property
-    def options(self):
+    def options(self) -> Dict[str, Any]:
         """Any non-default pool options that were set on this Connection Pool."""
         return self.__options
 
@@ -805,12 +842,12 @@ class PoolClearedEvent(_PoolEvent):
 
     __slots__ = ("__service_id",)
 
-    def __init__(self, address, service_id=None):
+    def __init__(self, address: _Address, service_id: Optional[ObjectId] = None) -> None:
         super(PoolClearedEvent, self).__init__(address)
         self.__service_id = service_id
 
     @property
-    def service_id(self):
+    def service_id(self) -> Optional[ObjectId]:
         """Connections with this service_id are cleared.
 
         When service_id is ``None``, all connections in the pool are cleared.
@@ -881,19 +918,19 @@ class _ConnectionEvent(object):
 
     __slots__ = ("__address", "__connection_id")
 
-    def __init__(self, address, connection_id):
+    def __init__(self, address: _Address, connection_id: int) -> None:
         self.__address = address
         self.__connection_id = connection_id
 
     @property
-    def address(self):
+    def address(self) -> _Address:
         """The address (host, port) pair of the server this connection is
         attempting to connect to.
         """
         return self.__address
 
     @property
-    def connection_id(self):
+    def connection_id(self) -> int:
         """The ID of the Connection."""
         return self.__connection_id
 
@@ -1007,19 +1044,19 @@ class ConnectionCheckOutFailedEvent(object):
 
     __slots__ = ("__address", "__reason")
 
-    def __init__(self, address, reason):
+    def __init__(self, address: _Address, reason: str) -> None:
         self.__address = address
         self.__reason = reason
 
     @property
-    def address(self):
+    def address(self) -> _Address:
         """The address (host, port) pair of the server this connection is
         attempting to connect to.
         """
         return self.__address
 
     @property
-    def reason(self):
+    def reason(self) -> str:
         """A reason explaining why connection check out failed.
 
         The reason must be one of the strings from the
@@ -1064,17 +1101,17 @@ class _ServerEvent(object):
 
     __slots__ = ("__server_address", "__topology_id")
 
-    def __init__(self, server_address, topology_id):
+    def __init__(self, server_address: _Address, topology_id: ObjectId) -> None:
         self.__server_address = server_address
         self.__topology_id = topology_id
 
     @property
-    def server_address(self):
+    def server_address(self) -> _Address:
         """The address (host, port) pair of the server"""
         return self.__server_address
 
     @property
-    def topology_id(self):
+    def topology_id(self) -> ObjectId:
         """A unique identifier for the topology this server is a part of."""
         return self.__topology_id
 
@@ -1094,19 +1131,24 @@ class ServerDescriptionChangedEvent(_ServerEvent):
 
     __slots__ = ("__previous_description", "__new_description")
 
-    def __init__(self, previous_description, new_description, *args):
+    def __init__(
+        self,
+        previous_description: "ServerDescription",
+        new_description: "ServerDescription",
+        *args: Any
+    ) -> None:
         super(ServerDescriptionChangedEvent, self).__init__(*args)
         self.__previous_description = previous_description
         self.__new_description = new_description
 
     @property
-    def previous_description(self):
+    def previous_description(self) -> "ServerDescription":
         """The previous
         :class:`~pymongo.server_description.ServerDescription`."""
         return self.__previous_description
 
     @property
-    def new_description(self):
+    def new_description(self) -> "ServerDescription":
         """The new
         :class:`~pymongo.server_description.ServerDescription`."""
         return self.__new_description
@@ -1143,11 +1185,11 @@ class TopologyEvent(object):
 
     __slots__ = "__topology_id"
 
-    def __init__(self, topology_id):
+    def __init__(self, topology_id: ObjectId) -> None:
         self.__topology_id = topology_id
 
     @property
-    def topology_id(self):
+    def topology_id(self) -> ObjectId:
         """A unique identifier for the topology this server is a part of."""
         return self.__topology_id
 
@@ -1163,19 +1205,24 @@ class TopologyDescriptionChangedEvent(TopologyEvent):
 
     __slots__ = ("__previous_description", "__new_description")
 
-    def __init__(self, previous_description, new_description, *args):
+    def __init__(
+        self,
+        previous_description: "TopologyDescription",
+        new_description: "TopologyDescription",
+        *args: Any
+    ) -> None:
         super(TopologyDescriptionChangedEvent, self).__init__(*args)
         self.__previous_description = previous_description
         self.__new_description = new_description
 
     @property
-    def previous_description(self):
+    def previous_description(self) -> "TopologyDescription":
         """The previous
         :class:`~pymongo.topology_description.TopologyDescription`."""
         return self.__previous_description
 
     @property
-    def new_description(self):
+    def new_description(self) -> "TopologyDescription":
         """The new
         :class:`~pymongo.topology_description.TopologyDescription`."""
         return self.__new_description
@@ -1212,11 +1259,11 @@ class _ServerHeartbeatEvent(object):
 
     __slots__ = "__connection_id"
 
-    def __init__(self, connection_id):
+    def __init__(self, connection_id: _Address) -> None:
         self.__connection_id = connection_id
 
     @property
-    def connection_id(self):
+    def connection_id(self) -> _Address:
         """The address (host, port) of the server this heartbeat was sent
         to."""
         return self.__connection_id
@@ -1242,24 +1289,26 @@ class ServerHeartbeatSucceededEvent(_ServerHeartbeatEvent):
 
     __slots__ = ("__duration", "__reply", "__awaited")
 
-    def __init__(self, duration, reply, connection_id, awaited=False):
+    def __init__(
+        self, duration: float, reply: Hello, connection_id: _Address, awaited: bool = False
+    ) -> None:
         super(ServerHeartbeatSucceededEvent, self).__init__(connection_id)
         self.__duration = duration
         self.__reply = reply
         self.__awaited = awaited
 
     @property
-    def duration(self):
+    def duration(self) -> float:
         """The duration of this heartbeat in microseconds."""
         return self.__duration
 
     @property
-    def reply(self):
+    def reply(self) -> Hello:
         """An instance of :class:`~pymongo.hello.Hello`."""
         return self.__reply
 
     @property
-    def awaited(self):
+    def awaited(self) -> bool:
         """Whether the heartbeat was awaited.
 
         If true, then :meth:`duration` reflects the sum of the round trip time
@@ -1287,24 +1336,26 @@ class ServerHeartbeatFailedEvent(_ServerHeartbeatEvent):
 
     __slots__ = ("__duration", "__reply", "__awaited")
 
-    def __init__(self, duration, reply, connection_id, awaited=False):
+    def __init__(
+        self, duration: float, reply: Exception, connection_id: _Address, awaited: bool = False
+    ) -> None:
         super(ServerHeartbeatFailedEvent, self).__init__(connection_id)
         self.__duration = duration
         self.__reply = reply
         self.__awaited = awaited
 
     @property
-    def duration(self):
+    def duration(self) -> float:
         """The duration of this heartbeat in microseconds."""
         return self.__duration
 
     @property
-    def reply(self):
+    def reply(self) -> Exception:
         """A subclass of :exc:`Exception`."""
         return self.__reply
 
     @property
-    def awaited(self):
+    def awaited(self) -> bool:
         """Whether the heartbeat was awaited.
 
         If true, then :meth:`duration` reflects the sum of the round trip time
