@@ -24,7 +24,7 @@ try:
     import pymongo.pyopenssl_context as _ssl
 except ImportError:
     try:
-        import pymongo.ssl_context as _ssl
+        import pymongo.ssl_context as _ssl  # type: ignore[no-redef]
     except ImportError:
         HAVE_SSL = False
 
@@ -34,7 +34,7 @@ if HAVE_SSL:
     # CPython ssl module constants to configure certificate verification
     # at a high level. This is legacy behavior, but requires us to
     # import the ssl module even if we're only using it for this purpose.
-    import ssl as _stdlibssl
+    import ssl as _stdlibssl  # noqa
     from ssl import CERT_NONE, CERT_REQUIRED
 
     HAS_SNI = _ssl.HAS_SNI
@@ -79,7 +79,7 @@ if HAVE_SSL:
             if _ssl.IS_PYOPENSSL:
                 raise ConfigurationError("tlsCRLFile cannot be used with PyOpenSSL")
             # Match the server's behavior.
-            ctx.verify_flags = getattr(_ssl, "VERIFY_CRL_CHECK_LEAF", 0)
+            setattr(ctx, "verify_flags", getattr(_ssl, "VERIFY_CRL_CHECK_LEAF", 0))  # noqa
             ctx.load_verify_locations(crlfile)
         if ca_certs is not None:
             ctx.load_verify_locations(ca_certs)
@@ -90,12 +90,12 @@ if HAVE_SSL:
 
 else:
 
-    class SSLError(Exception):
+    class SSLError(Exception):  # type: ignore
         pass
 
     HAS_SNI = False
     IPADDR_SAFE = False
 
-    def get_ssl_context(*dummy):
+    def get_ssl_context(*dummy):  # type: ignore
         """No ssl module, raise ConfigurationError."""
         raise ConfigurationError("The ssl module is not available.")
