@@ -21,6 +21,7 @@ import threading
 import time
 import warnings
 import weakref
+from typing import Any
 
 from pymongo import common, helpers, periodic_executor
 from pymongo.client_session import _ServerSessionPool
@@ -86,12 +87,13 @@ class Topology(object):
 
         # Create events queue if there are publishers.
         self._events = None
-        self.__events_executor = None
+        self.__events_executor: Any = None
 
         if self._publish_server or self._publish_tp:
             self._events = queue.Queue(maxsize=100)
 
         if self._publish_tp:
+            assert self._events is not None
             self._events.put((self._listeners.publish_topology_opened, (self._topology_id,)))
         self._settings = topology_settings
         topology_description = TopologyDescription(
@@ -105,6 +107,7 @@ class Topology(object):
 
         self._description = topology_description
         if self._publish_tp:
+            assert self._events is not None
             initial_td = TopologyDescription(
                 TOPOLOGY_TYPE.Unknown, {}, None, None, None, self._settings
             )
@@ -117,6 +120,7 @@ class Topology(object):
 
         for seed in topology_settings.seeds:
             if self._publish_server:
+                assert self._events is not None
                 self._events.put((self._listeners.publish_server_opened, (seed, self._topology_id)))
 
         # Store the seed list to help diagnose errors in _error_message().
@@ -295,6 +299,7 @@ class Topology(object):
 
         suppress_event = (self._publish_server or self._publish_tp) and sd_old == server_description
         if self._publish_server and not suppress_event:
+            assert self._events is not None
             self._events.put(
                 (
                     self._listeners.publish_server_description_changed,
@@ -307,6 +312,7 @@ class Topology(object):
         self._receive_cluster_time_no_lock(server_description.cluster_time)
 
         if self._publish_tp and not suppress_event:
+            assert self._events is not None
             self._events.put(
                 (
                     self._listeners.publish_topology_description_changed,
@@ -357,6 +363,7 @@ class Topology(object):
         self._update_servers()
 
         if self._publish_tp:
+            assert self._events is not None
             self._events.put(
                 (
                     self._listeners.publish_topology_description_changed,
@@ -453,7 +460,7 @@ class Topology(object):
             return self._description.known_servers
         return self._description.readable_servers
 
-    def update_pool(self, all_credentials):
+    def update_pool(self):
         # Remove any stale sockets and add new sockets if pool is too small.
         servers = []
         with self._lock:
@@ -464,7 +471,7 @@ class Topology(object):
 
         for server, generation in servers:
             try:
-                server.pool.remove_stale_sockets(generation, all_credentials)
+                server.pool.remove_stale_sockets(generation)
             except PyMongoError as exc:
                 ctx = _ErrorContext(exc, 0, generation, False, None)
                 self.handle_error(server.description.address, ctx)
@@ -493,6 +500,7 @@ class Topology(object):
 
         # Publish only after releasing the lock.
         if self._publish_tp:
+            assert self._events is not None
             self._events.put((self._listeners.publish_topology_closed, (self._topology_id,)))
         if self._publish_server or self._publish_tp:
             self.__events_executor.close()
@@ -506,8 +514,15 @@ class Topology(object):
         with self._lock:
             return self._session_pool.pop_all()
 
+    def _check_implicit_session_support(self):
+        with self._lock:
+            self._check_session_support()
+
     def _check_session_support(self):
-        """Internal check for session support on non-load balanced clusters."""
+        """Internal check for session support on clusters."""
+        if self._settings.load_balanced:
+            # Sessions never time out in load balanced mode.
+            return float("inf")
         session_timeout = self._description.logical_session_timeout_minutes
         if session_timeout is None:
             # Maybe we need an initial scan? Can raise ServerSelectionError.
@@ -529,12 +544,7 @@ class Topology(object):
     def get_server_session(self):
         """Start or resume a server session, or raise ConfigurationError."""
         with self._lock:
-            # Sessions are always supported in load balanced mode.
-            if not self._settings.load_balanced:
-                session_timeout = self._check_session_support()
-            else:
-                # Sessions never time out in load balanced mode.
-                session_timeout = float("inf")
+            session_timeout = self._check_session_support()
             return self._session_pool.get_server_session(session_timeout)
 
     def return_server_session(self, server_session, lock):
