@@ -14,43 +14,48 @@
 
 """Represent a deployment of MongoDB servers."""
 
-from collections import namedtuple
 from random import sample
+from typing import Any, Callable, Dict, List, NamedTuple, Optional, Tuple
 
+from bson.objectid import ObjectId
 from pymongo import common
 from pymongo.errors import ConfigurationError
-from pymongo.read_preferences import ReadPreference
+from pymongo.read_preferences import ReadPreference, _AggWritePref, _ServerMode
 from pymongo.server_description import ServerDescription
 from pymongo.server_selectors import Selection
 from pymongo.server_type import SERVER_TYPE
+from pymongo.typings import _Address
+
 
 # Enumeration for various kinds of MongoDB cluster topologies.
-TOPOLOGY_TYPE = namedtuple(
-    "TopologyType",
-    [
-        "Single",
-        "ReplicaSetNoPrimary",
-        "ReplicaSetWithPrimary",
-        "Sharded",
-        "Unknown",
-        "LoadBalanced",
-    ],
-)(*range(6))
+class _TopologyType(NamedTuple):
+    Single: int
+    ReplicaSetNoPrimary: int
+    ReplicaSetWithPrimary: int
+    Sharded: int
+    Unknown: int
+    LoadBalanced: int
+
+
+TOPOLOGY_TYPE = _TopologyType(*range(6))
 
 # Topologies compatible with SRV record polling.
-SRV_POLLING_TOPOLOGIES = (TOPOLOGY_TYPE.Unknown, TOPOLOGY_TYPE.Sharded)
+SRV_POLLING_TOPOLOGIES: Tuple[int, int] = (TOPOLOGY_TYPE.Unknown, TOPOLOGY_TYPE.Sharded)
+
+
+_ServerSelector = Callable[[List[ServerDescription]], List[ServerDescription]]
 
 
 class TopologyDescription(object):
     def __init__(
         self,
-        topology_type,
-        server_descriptions,
-        replica_set_name,
-        max_set_version,
-        max_election_id,
-        topology_settings,
-    ):
+        topology_type: int,
+        server_descriptions: Dict[_Address, ServerDescription],
+        replica_set_name: Optional[str],
+        max_set_version: Optional[int],
+        max_election_id: Optional[ObjectId],
+        topology_settings: Any,
+    ) -> None:
         """Representation of a deployment of MongoDB servers.
 
         :Parameters:
@@ -89,7 +94,7 @@ class TopologyDescription(object):
         elif any(s.logical_session_timeout_minutes is None for s in readable_servers):
             self._ls_timeout_minutes = None
         else:
-            self._ls_timeout_minutes = min(
+            self._ls_timeout_minutes = min(  # type: ignore[type-var]
                 s.logical_session_timeout_minutes for s in readable_servers
             )
 
@@ -115,11 +120,11 @@ class TopologyDescription(object):
 
             if server_too_new:
                 self._incompatible_err = (
-                    "Server at %s:%d requires wire version %d, but this "
+                    "Server at %s:%d requires wire version %d, but this "  # type: ignore
                     "version of PyMongo only supports up to %d."
                     % (
                         s.address[0],
-                        s.address[1],
+                        s.address[1] or 0,
                         s.min_wire_version,
                         common.MAX_SUPPORTED_WIRE_VERSION,
                     )
@@ -127,11 +132,11 @@ class TopologyDescription(object):
 
             elif server_too_old:
                 self._incompatible_err = (
-                    "Server at %s:%d reports wire version %d, but this "
+                    "Server at %s:%d reports wire version %d, but this "  # type: ignore
                     "version of PyMongo requires at least %d (MongoDB %s)."
                     % (
                         s.address[0],
-                        s.address[1],
+                        s.address[1] or 0,
                         s.max_wire_version,
                         common.MIN_SUPPORTED_WIRE_VERSION,
                         common.MIN_SUPPORTED_SERVER_VERSION,
@@ -140,7 +145,7 @@ class TopologyDescription(object):
 
                 break
 
-    def check_compatible(self):
+    def check_compatible(self) -> None:
         """Raise ConfigurationError if any server is incompatible.
 
         A server is incompatible if its wire protocol version range does not
@@ -149,15 +154,15 @@ class TopologyDescription(object):
         if self._incompatible_err:
             raise ConfigurationError(self._incompatible_err)
 
-    def has_server(self, address):
+    def has_server(self, address: _Address) -> bool:
         return address in self._server_descriptions
 
-    def reset_server(self, address):
+    def reset_server(self, address: _Address) -> "TopologyDescription":
         """A copy of this description, with one server marked Unknown."""
         unknown_sd = self._server_descriptions[address].to_unknown()
         return updated_topology_description(self, unknown_sd)
 
-    def reset(self):
+    def reset(self) -> "TopologyDescription":
         """A copy of this description, with all servers marked Unknown."""
         if self._topology_type == TOPOLOGY_TYPE.ReplicaSetWithPrimary:
             topology_type = TOPOLOGY_TYPE.ReplicaSetNoPrimary
@@ -176,18 +181,18 @@ class TopologyDescription(object):
             self._topology_settings,
         )
 
-    def server_descriptions(self):
+    def server_descriptions(self) -> Dict[_Address, ServerDescription]:
         """Dict of (address,
         :class:`~pymongo.server_description.ServerDescription`)."""
         return self._server_descriptions.copy()
 
     @property
-    def topology_type(self):
+    def topology_type(self) -> int:
         """The type of this topology."""
         return self._topology_type
 
     @property
-    def topology_type_name(self):
+    def topology_type_name(self) -> str:
         """The topology type as a human readable string.
 
         .. versionadded:: 3.4
@@ -195,42 +200,42 @@ class TopologyDescription(object):
         return TOPOLOGY_TYPE._fields[self._topology_type]
 
     @property
-    def replica_set_name(self):
+    def replica_set_name(self) -> Optional[str]:
         """The replica set name."""
         return self._replica_set_name
 
     @property
-    def max_set_version(self):
+    def max_set_version(self) -> Optional[int]:
         """Greatest setVersion seen from a primary, or None."""
         return self._max_set_version
 
     @property
-    def max_election_id(self):
+    def max_election_id(self) -> Optional[ObjectId]:
         """Greatest electionId seen from a primary, or None."""
         return self._max_election_id
 
     @property
-    def logical_session_timeout_minutes(self):
+    def logical_session_timeout_minutes(self) -> Optional[int]:
         """Minimum logical session timeout, or None."""
         return self._ls_timeout_minutes
 
     @property
-    def known_servers(self):
+    def known_servers(self) -> List[ServerDescription]:
         """List of Servers of types besides Unknown."""
         return [s for s in self._server_descriptions.values() if s.is_server_type_known]
 
     @property
-    def has_known_servers(self):
+    def has_known_servers(self) -> bool:
         """Whether there are any Servers of types besides Unknown."""
         return any(s for s in self._server_descriptions.values() if s.is_server_type_known)
 
     @property
-    def readable_servers(self):
+    def readable_servers(self) -> List[ServerDescription]:
         """List of readable Servers."""
         return [s for s in self._server_descriptions.values() if s.is_readable]
 
     @property
-    def common_wire_version(self):
+    def common_wire_version(self) -> Optional[int]:
         """Minimum of all servers' max wire versions, or None."""
         servers = self.known_servers
         if servers:
@@ -239,11 +244,11 @@ class TopologyDescription(object):
         return None
 
     @property
-    def heartbeat_frequency(self):
+    def heartbeat_frequency(self) -> int:
         return self._topology_settings.heartbeat_frequency
 
     @property
-    def srv_max_hosts(self):
+    def srv_max_hosts(self) -> int:
         return self._topology_settings._srv_max_hosts
 
     def _apply_local_threshold(self, selection):
@@ -256,7 +261,12 @@ class TopologyDescription(object):
             s for s in selection.server_descriptions if (s.round_trip_time - fastest) <= threshold
         ]
 
-    def apply_selector(self, selector, address=None, custom_selector=None):
+    def apply_selector(
+        self,
+        selector: Any,
+        address: Optional[_Address] = None,
+        custom_selector: Optional[_ServerSelector] = None,
+    ) -> List[ServerDescription]:
         """List of servers matching the provided selector(s).
 
         :Parameters:
@@ -280,20 +290,23 @@ class TopologyDescription(object):
                     " wire version is %d" % (selector, selector.min_wire_version, common_wv)
                 )
 
+        if isinstance(selector, _AggWritePref):
+            selector.selection_hook(self)
+
         if self.topology_type == TOPOLOGY_TYPE.Unknown:
             return []
         elif self.topology_type in (TOPOLOGY_TYPE.Single, TOPOLOGY_TYPE.LoadBalanced):
             # Ignore selectors for standalone and load balancer mode.
             return self.known_servers
-        elif address:
+        if address:
             # Ignore selectors when explicit address is requested.
             description = self.server_descriptions().get(address)
             return [description] if description else []
-        elif self.topology_type == TOPOLOGY_TYPE.Sharded:
-            # Ignore read preference.
-            selection = Selection.from_topology_description(self)
-        else:
-            selection = selector(Selection.from_topology_description(self))
+
+        selection = Selection.from_topology_description(self)
+        # Ignore read preference for sharded clusters.
+        if self.topology_type != TOPOLOGY_TYPE.Sharded:
+            selection = selector(selection)
 
         # Apply custom selector followed by localThresholdMS.
         if custom_selector is not None and selection:
@@ -302,7 +315,7 @@ class TopologyDescription(object):
             )
         return self._apply_local_threshold(selection)
 
-    def has_readable_server(self, read_preference=ReadPreference.PRIMARY):
+    def has_readable_server(self, read_preference: _ServerMode = ReadPreference.PRIMARY) -> bool:
         """Does this topology have any readable servers available matching the
         given read preference?
 
@@ -319,7 +332,7 @@ class TopologyDescription(object):
         common.validate_read_preference("read_preference", read_preference)
         return any(self.apply_selector(read_preference))
 
-    def has_writable_server(self):
+    def has_writable_server(self) -> bool:
         """Does this topology have a writable server available?
 
         .. note:: When connected directly to a single server this method
@@ -352,7 +365,9 @@ _SERVER_TYPE_TO_TOPOLOGY_TYPE = {
 }
 
 
-def updated_topology_description(topology_description, server_description):
+def updated_topology_description(
+    topology_description: TopologyDescription, server_description: ServerDescription
+) -> "TopologyDescription":
     """Return an updated copy of a TopologyDescription.
 
     :Parameters:
