@@ -20,7 +20,6 @@ import os
 import socket
 import sys
 import threading
-import time
 import traceback
 import unittest
 import warnings
@@ -34,7 +33,7 @@ except (ImportError, ValueError):
     HAVE_XML = False
 
 try:
-    import ipaddress
+    import ipaddress  # noqa
 
     HAVE_IPADDRESS = True
 except ImportError:
@@ -43,14 +42,18 @@ except ImportError:
 from contextlib import contextmanager
 from functools import wraps
 from test.version import Version
+from typing import Dict, no_type_check
 from unittest import SkipTest
+from urllib.parse import quote_plus
 
 import pymongo
 import pymongo.errors
 from bson.son import SON
 from pymongo import common, message
 from pymongo.common import partition_node
+from pymongo.database import Database
 from pymongo.hello import HelloCompat
+from pymongo.mongo_client import MongoClient
 from pymongo.server_api import ServerApi
 from pymongo.ssl_support import HAVE_SSL, _ssl
 from pymongo.uri_parser import parse_uri
@@ -85,7 +88,7 @@ CERT_PATH = os.path.join(os.path.dirname(os.path.realpath(__file__)), "certifica
 CLIENT_PEM = os.environ.get("CLIENT_PEM", os.path.join(CERT_PATH, "client.pem"))
 CA_PEM = os.environ.get("CA_PEM", os.path.join(CERT_PATH, "ca.pem"))
 
-TLS_OPTIONS = dict(tls=True)
+TLS_OPTIONS: Dict = dict(tls=True)
 if CLIENT_PEM:
     TLS_OPTIONS["tlsCertificateKeyFile"] = CLIENT_PEM
 if CA_PEM:
@@ -104,7 +107,7 @@ if TEST_LOADBALANCER:
     db_pwd = res["password"] or db_pwd
 elif TEST_SERVERLESS:
     TEST_LOADBALANCER = True
-    res = parse_uri(SINGLE_MONGOS_LB_URI)
+    res = parse_uri(SINGLE_MONGOS_LB_URI or "")
     host, port = res["nodelist"][0]
     db_user = res["username"] or db_user
     db_pwd = res["password"] or db_pwd
@@ -181,6 +184,7 @@ class client_knobs(object):
     def __enter__(self):
         self.enable()
 
+    @no_type_check
     def disable(self):
         common.HEARTBEAT_FREQUENCY = self.old_heartbeat_frequency
         common.MIN_HEARTBEAT_INTERVAL = self.old_min_heartbeat_interval
@@ -225,6 +229,8 @@ def _all_users(db):
 
 
 class ClientContext(object):
+    client: MongoClient
+
     MULTI_MONGOS_LB_URI = MULTI_MONGOS_LB_URI
 
     def __init__(self):
@@ -248,9 +254,9 @@ class ClientContext(object):
         self.tls = False
         self.tlsCertificateKeyFile = False
         self.server_is_resolvable = is_server_resolvable()
-        self.default_client_options = {}
+        self.default_client_options: Dict = {}
         self.sessions_enabled = False
-        self.client = None
+        self.client = None  # type: ignore
         self.conn_lock = threading.Lock()
         self.is_data_lake = False
         self.load_balancer = TEST_LOADBALANCER
@@ -274,6 +280,23 @@ class ClientContext(object):
             opts["replicaSet"] = self.replica_set_name
         return opts
 
+    @property
+    def uri(self):
+        """Return the MongoClient URI for creating a duplicate client."""
+        opts = client_context.default_client_options.copy()
+        opts.pop("server_api", None)  # Cannot be set from the URI
+        opts_parts = []
+        for opt, val in opts.items():
+            strval = str(val)
+            if isinstance(val, bool):
+                strval = strval.lower()
+            opts_parts.append(f"{opt}={quote_plus(strval)}")
+        opts_part = "&".join(opts_parts)
+        auth_part = ""
+        if client_context.auth_enabled:
+            auth_part = f"{quote_plus(db_user)}:{quote_plus(db_pwd)}@"
+        return f"mongodb://{auth_part}{self.pair}/?{opts_part}"
+
     @property
     def hello(self):
         if not self._hello:
@@ -281,13 +304,8 @@ class ClientContext(object):
         return self._hello
 
     def _connect(self, host, port, **kwargs):
-        # Jython takes a long time to connect.
-        if sys.platform.startswith("java"):
-            timeout_ms = 10000
-        else:
-            timeout_ms = 5000
         kwargs.update(self.default_client_options)
-        client = pymongo.MongoClient(host, port, serverSelectionTimeoutMS=timeout_ms, **kwargs)
+        client = pymongo.MongoClient(host, port, serverSelectionTimeoutMS=5000, **kwargs)
         try:
             try:
                 client.admin.command(HelloCompat.LEGACY_CMD)  # Can we connect?
@@ -337,6 +355,7 @@ class ClientContext(object):
                 try:
                     self.cmd_line = self.client.admin.command("getCmdLineOpts")
                 except pymongo.errors.OperationFailure as e:
+                    assert e.details is not None
                     msg = e.details.get("errmsg", "")
                     if e.code == 13 or "unauthorized" in msg or "login" in msg:
                         # Unauthorized.
@@ -358,7 +377,7 @@ class ClientContext(object):
                     username=db_user,
                     password=db_pwd,
                     replicaSet=self.replica_set_name,
-                    **self.default_client_options
+                    **self.default_client_options,
                 )
 
                 # May not have this if OperationFailure was raised earlier.
@@ -386,7 +405,7 @@ class ClientContext(object):
                         username=db_user,
                         password=db_pwd,
                         replicaSet=self.replica_set_name,
-                        **self.default_client_options
+                        **self.default_client_options,
                     )
                 else:
                     self.client = pymongo.MongoClient(
@@ -414,6 +433,7 @@ class ClientContext(object):
                 self.has_ipv6 = False
             else:
                 self.server_parameters = self.client.admin.command("getParameter", "*")
+                assert self.cmd_line is not None
                 if "enableTestCommands=1" in self.cmd_line["argv"]:
                     self.test_commands_enabled = True
                 elif "parsed" in self.cmd_line:
@@ -432,6 +452,7 @@ class ClientContext(object):
                 self.mongoses.append(address)
                 if not self.serverless:
                     # Check for another mongos on the next port.
+                    assert address is not None
                     next_address = address[0], address[1] + 1
                     mongos_client = self._connect(*next_address, **self.default_client_options)
                     if mongos_client:
@@ -487,12 +508,13 @@ class ClientContext(object):
             username=db_user,
             password=db_pwd,
             serverSelectionTimeoutMS=100,
-            **self.default_client_options
+            **self.default_client_options,
         )
 
         try:
             return db_user in _all_users(client.admin)
         except pymongo.errors.OperationFailure as e:
+            assert e.details is not None
             msg = e.details.get("errmsg", "")
             if e.code == 18 or "auth fails" in msg:
                 # Auth failed.
@@ -502,6 +524,7 @@ class ClientContext(object):
 
     def _server_started_with_auth(self):
         # MongoDB >= 2.0
+        assert self.cmd_line is not None
         if "parsed" in self.cmd_line:
             parsed = self.cmd_line["parsed"]
             # MongoDB >= 2.6
@@ -521,6 +544,7 @@ class ClientContext(object):
         if not socket.has_ipv6:
             return False
 
+        assert self.cmd_line is not None
         if "parsed" in self.cmd_line:
             if not self.cmd_line["parsed"].get("net", {}).get("ipv6"):
                 return False
@@ -655,7 +679,7 @@ class ClientContext(object):
         """
         return self._require(
             lambda: self.supports_secondary_read_pref,
-            "This cluster does not support secondary read " "preference",
+            "This cluster does not support secondary read preference",
         )
 
     def require_no_replica_set(self, func):
@@ -745,7 +769,7 @@ class ClientContext(object):
             return True
         return False
 
-    def require_cluster_type(self, topologies=[]):
+    def require_cluster_type(self, topologies=[]):  # noqa
         """Run a test only if the client is connected to a cluster that
         conforms to one of the specified topologies. Acceptable topologies
         are 'single', 'replicaset', and 'sharded'."""
@@ -813,7 +837,7 @@ class ClientContext(object):
         """Run a test only if the hostname 'server' is resolvable."""
         return self._require(
             lambda: self.server_is_resolvable,
-            "No hosts entry for 'server'. Cannot validate " "hostname in the certificate",
+            "No hosts entry for 'server'. Cannot validate hostname in the certificate",
             func=func,
         )
 
@@ -902,7 +926,7 @@ def sanitize_cmd(cmd):
     cp.pop("$readPreference", None)
     cp.pop("lsid", None)
     if MONGODB_API_VERSION:
-        # Versioned api parameters
+        # Stable API parameters
         cp.pop("apiVersion", None)
     # OP_MSG encoding may move the payload type one field to the
     # end of the command. Do the same here.
@@ -946,6 +970,10 @@ class PyMongoTestCase(unittest.TestCase):
 class IntegrationTest(PyMongoTestCase):
     """Base class for TestCases that need a connection to MongoDB to pass."""
 
+    client: MongoClient
+    db: Database
+    credentials: Dict[str, str]
+
     @classmethod
     @client_context.require_connection
     def setUpClass(cls):
@@ -1022,21 +1050,26 @@ def _get_executors(topology):
     return [e for e in executors if e is not None]
 
 
-def all_executors_stopped(topology):
+def print_running_topology(topology):
     running = [e for e in _get_executors(topology) if not e._stopped]
     if running:
         print(
-            "  Topology %s has THREADS RUNNING: %s, created at: %s"
-            % (topology, running, topology._settings._stack)
+            "WARNING: found Topology with running threads:\n"
+            "  Threads: %s\n"
+            "  Topology: %s\n"
+            "  Creation traceback:\n%s" % (running, topology, topology._settings._stack)
         )
-        return False
-    return True
 
 
-def print_unclosed_clients():
+def print_running_clients():
     from pymongo.topology import Topology
 
     processed = set()
+    # Avoid false positives on the main test client.
+    # XXX: Can be removed after PYTHON-1634 or PYTHON-1896.
+    c = client_context.client
+    if c:
+        processed.add(c._topology._topology_id)
     # Call collect to manually cleanup any would-be gc'd clients to avoid
     # false positives.
     gc.collect()
@@ -1046,7 +1079,7 @@ def print_unclosed_clients():
                 # Avoid printing the same Topology multiple times.
                 if obj._topology_id in processed:
                     continue
-                all_executors_stopped(obj)
+                print_running_topology(obj)
                 processed.add(obj._topology_id)
         except ReferenceError:
             pass
@@ -1071,9 +1104,7 @@ def teardown():
             c.drop_database("pymongo_test_bernie")
         c.close()
 
-    # Jython does not support gc.get_objects.
-    if not sys.platform.startswith("java"):
-        print_unclosed_clients()
+    print_running_clients()
 
 
 class PymongoTestRunner(unittest.TextTestRunner):
@@ -1086,7 +1117,7 @@ class PymongoTestRunner(unittest.TextTestRunner):
 
 if HAVE_XML:
 
-    class PymongoXMLTestRunner(XMLTestRunner):
+    class PymongoXMLTestRunner(XMLTestRunner):  # type: ignore[misc]
         def run(self, test):
             setup()
             result = super(PymongoXMLTestRunner, self).run(test)
@@ -1109,9 +1140,9 @@ def test_cases(suite):
 # Helper method to workaround https://bugs.python.org/issue21724
 def clear_warning_registry():
     """Clear the __warningregistry__ for all modules."""
-    for name, module in list(sys.modules.items()):
+    for _, module in list(sys.modules.items()):
         if hasattr(module, "__warningregistry__"):
-            setattr(module, "__warningregistry__", {})
+            setattr(module, "__warningregistry__", {})  # noqa
 
 
 class SystemCertsPatcher(object):
