@@ -21,7 +21,7 @@ from urllib.parse import quote_plus
 
 sys.path[0:0] = [""]
 
-from test import IntegrationTest, SkipTest, Version, client_context, unittest
+from test import IntegrationTest, SkipTest, client_context, unittest
 from test.utils import (
     AllowListEventListener,
     delay,
@@ -77,6 +77,9 @@ class AutoAuthenticateThread(threading.Thread):
 
 
 class TestGSSAPI(unittest.TestCase):
+    mech_properties: str
+    service_realm_required: bool
+
     @classmethod
     def setUpClass(cls):
         if not HAVE_KERBEROS:
@@ -113,15 +116,16 @@ class TestGSSAPI(unittest.TestCase):
 
     @ignore_deprecations
     def test_gssapi_simple(self):
+        assert GSSAPI_PRINCIPAL is not None
         if GSSAPI_PASS is not None:
-            uri = "mongodb://%s:%s@%s:%d/?authMechanism=" "GSSAPI" % (
+            uri = "mongodb://%s:%s@%s:%d/?authMechanism=GSSAPI" % (
                 quote_plus(GSSAPI_PRINCIPAL),
                 GSSAPI_PASS,
                 GSSAPI_HOST,
                 GSSAPI_PORT,
             )
         else:
-            uri = "mongodb://%s@%s:%d/?authMechanism=" "GSSAPI" % (
+            uri = "mongodb://%s@%s:%d/?authMechanism=GSSAPI" % (
                 quote_plus(GSSAPI_PRINCIPAL),
                 GSSAPI_HOST,
                 GSSAPI_PORT,
@@ -261,7 +265,7 @@ class TestSASLPlain(unittest.TestCase):
     @classmethod
     def setUpClass(cls):
         if not SASL_HOST or not SASL_USER or not SASL_PASS:
-            raise SkipTest("Must set SASL_HOST, " "SASL_USER, and SASL_PASS to test SASL")
+            raise SkipTest("Must set SASL_HOST, SASL_USER, and SASL_PASS to test SASL")
 
     def test_sasl_plain(self):
 
@@ -275,7 +279,9 @@ class TestSASLPlain(unittest.TestCase):
         )
         client.ldap.test.find_one()
 
-        uri = "mongodb://%s:%s@%s:%d/?authMechanism=PLAIN;" "authSource=%s" % (
+        assert SASL_USER is not None
+        assert SASL_PASS is not None
+        uri = "mongodb://%s:%s@%s:%d/?authMechanism=PLAIN;authSource=%s" % (
             quote_plus(SASL_USER),
             quote_plus(SASL_PASS),
             SASL_HOST,
@@ -298,7 +304,7 @@ class TestSASLPlain(unittest.TestCase):
             )
             client.ldap.test.find_one()
 
-            uri = "mongodb://%s:%s@%s:%d/?authMechanism=PLAIN;" "authSource=%s;replicaSet=%s" % (
+            uri = "mongodb://%s:%s@%s:%d/?authMechanism=PLAIN;authSource=%s;replicaSet=%s" % (
                 quote_plus(SASL_USER),
                 quote_plus(SASL_PASS),
                 SASL_HOST,
@@ -311,7 +317,7 @@ class TestSASLPlain(unittest.TestCase):
 
     def test_sasl_plain_bad_credentials(self):
         def auth_string(user, password):
-            uri = "mongodb://%s:%s@%s:%d/?authMechanism=PLAIN;" "authSource=%s" % (
+            uri = "mongodb://%s:%s@%s:%d/?authMechanism=PLAIN;authSource=%s" % (
                 quote_plus(user),
                 quote_plus(password),
                 SASL_HOST,
@@ -477,7 +483,7 @@ class TestSCRAM(IntegrationTest):
 
         if client_context.is_rs:
             host, port = client_context.host, client_context.port
-            uri = "mongodb://both:pwd@%s:%d/testscram" "?replicaSet=%s" % (
+            uri = "mongodb://both:pwd@%s:%d/testscram?replicaSet=%s" % (
                 host,
                 port,
                 client_context.replica_set_name,
@@ -552,10 +558,12 @@ class TestSCRAM(IntegrationTest):
 
     def test_cache(self):
         client = single_client()
+        credentials = client.options.pool_options._credentials
+        cache = credentials.cache
+        self.assertIsNotNone(cache)
+        self.assertIsNone(cache.data)
         # Force authentication.
         client.admin.command("ping")
-        all_credentials = client._MongoClient__all_credentials
-        credentials = all_credentials.get("admin")
         cache = credentials.cache
         self.assertIsNotNone(cache)
         data = cache.data
@@ -567,21 +575,7 @@ class TestSCRAM(IntegrationTest):
         self.assertIsInstance(salt, bytes)
         self.assertIsInstance(iterations, int)
 
-        pool = next(iter(client._topology._servers.values()))._pool
-        with pool.get_socket(all_credentials) as sock_info:
-            authset = sock_info.authset
-        cached = set(all_credentials.values())
-        self.assertEqual(len(cached), 1)
-        self.assertFalse(authset - cached)
-        self.assertFalse(cached - authset)
-
-        sock_credentials = next(iter(authset))
-        sock_cache = sock_credentials.cache
-        self.assertIsNotNone(sock_cache)
-        self.assertEqual(sock_cache.data, data)
-
     def test_scram_threaded(self):
-
         coll = client_context.client.db.test
         coll.drop()
         coll.insert_one({"_id": 1})
@@ -646,7 +640,7 @@ class TestAuthURIOptions(IntegrationTest):
             self.assertTrue(db.command("dbstats"))
 
         # Test authSource
-        uri = "mongodb://user:pass@%s:%d" "/pymongo_test2?authSource=pymongo_test" % (host, port)
+        uri = "mongodb://user:pass@%s:%d/pymongo_test2?authSource=pymongo_test" % (host, port)
         client = rs_or_single_client_noauth(uri)
         self.assertRaises(OperationFailure, client.pymongo_test2.command, "dbstats")
         self.assertTrue(client.pymongo_test.command("dbstats"))
