@@ -23,10 +23,11 @@ import os
 import signal
 import socket
 import struct
+import subprocess
 import sys
 import threading
 import time
-import warnings
+from typing import Iterable, Type, no_type_check
 
 sys.path[0:0] = [""]
 
@@ -87,7 +88,6 @@ from pymongo.errors import (
     ServerSelectionTimeoutError,
     WriteConcernError,
 )
-from pymongo.hello import HelloCompat
 from pymongo.mongo_client import MongoClient
 from pymongo.monitoring import ServerHeartbeatListener, ServerHeartbeatStartedEvent
 from pymongo.pool import _METADATA, PoolOptions, SocketInfo
@@ -98,16 +98,15 @@ from pymongo.server_type import SERVER_TYPE
 from pymongo.settings import TOPOLOGY_TYPE
 from pymongo.srv_resolver import _HAVE_DNSPYTHON
 from pymongo.topology import _ErrorContext
-from pymongo.topology_description import (
-    TopologyDescription,
-    _updated_topology_description_srv_polling,
-)
+from pymongo.topology_description import TopologyDescription
 from pymongo.write_concern import WriteConcern
 
 
 class ClientUnitTest(unittest.TestCase):
     """MongoClient tests that don't require a server."""
 
+    client: MongoClient
+
     @classmethod
     @client_context.require_connection
     def setUpClass(cls):
@@ -212,10 +211,26 @@ class ClientUnitTest(unittest.TestCase):
         self.assertIn("has no attribute '_does_not_exist'", str(context.exception))
 
     def test_iteration(self):
-        def iterate():
-            [a for a in self.client]
-
-        self.assertRaises(TypeError, iterate)
+        client = self.client
+        if "PyPy" in sys.version:
+            msg = "'NoneType' object is not callable"
+        else:
+            msg = "'MongoClient' object is not iterable"
+        # Iteration fails
+        with self.assertRaisesRegex(TypeError, msg):
+            for _ in client:  # type: ignore[misc] # error: "None" not callable  [misc]
+                break
+        # Index fails
+        with self.assertRaises(TypeError):
+            _ = client[0]
+        # next fails
+        with self.assertRaisesRegex(TypeError, "'MongoClient' object is not iterable"):
+            _ = next(client)
+        # .next() fails
+        with self.assertRaisesRegex(TypeError, "'MongoClient' object is not iterable"):
+            _ = client.next()
+        # Do not implement typing.Iterable.
+        self.assertNotIsInstance(client, Iterable)
 
     def test_get_default_database(self):
         c = rs_or_single_client(
@@ -276,7 +291,7 @@ class ClientUnitTest(unittest.TestCase):
             MongoClient("mongodb://host/?readpreferencetags=dc:east")
 
         with self.assertRaises(ConfigurationError):
-            MongoClient("mongodb://host/?" "readpreference=primary&readpreferencetags=dc:east")
+            MongoClient("mongodb://host/?readpreference=primary&readpreferencetags=dc:east")
 
     def test_read_preference(self):
         c = rs_or_single_client(
@@ -342,7 +357,7 @@ class ClientUnitTest(unittest.TestCase):
                 return int(value)
 
         # Ensure codec options are passed in correctly
-        document_class = SON
+        document_class: Type[SON] = SON
         type_registry = TypeRegistry([MyFloatAsIntEncoder()])
         tz_aware = True
         uuid_representation_label = "javaLegacy"
@@ -391,7 +406,7 @@ class ClientUnitTest(unittest.TestCase):
 
     def test_uri_option_precedence(self):
         # Ensure kwarg options override connection string options.
-        uri = "mongodb://localhost/?ssl=true&replicaSet=name" "&readPreference=primary"
+        uri = "mongodb://localhost/?ssl=true&replicaSet=name&readPreference=primary"
         c = MongoClient(uri, ssl=False, replicaSet="newname", readPreference="secondaryPreferred")
         clopts = c._MongoClient__options
         opts = clopts._options
@@ -507,7 +522,7 @@ class TestClient(IntegrationTest):
             # Assert reaper doesn't remove sockets when maxIdleTimeMS not set
             client = rs_or_single_client()
             server = client._get_topology().select_server(readable_server_selector)
-            with server._pool.get_socket({}) as sock_info:
+            with server._pool.get_socket() as sock_info:
                 pass
             self.assertEqual(1, len(server._pool.sockets))
             self.assertTrue(sock_info in server._pool.sockets)
@@ -518,7 +533,7 @@ class TestClient(IntegrationTest):
             # Assert reaper removes idle socket and replaces it with a new one
             client = rs_or_single_client(maxIdleTimeMS=500, minPoolSize=1)
             server = client._get_topology().select_server(readable_server_selector)
-            with server._pool.get_socket({}) as sock_info:
+            with server._pool.get_socket() as sock_info:
                 pass
             # When the reaper runs at the same time as the get_socket, two
             # sockets could be created and checked into the pool.
@@ -532,7 +547,7 @@ class TestClient(IntegrationTest):
             # Assert reaper respects maxPoolSize when adding new sockets.
             client = rs_or_single_client(maxIdleTimeMS=500, minPoolSize=1, maxPoolSize=1)
             server = client._get_topology().select_server(readable_server_selector)
-            with server._pool.get_socket({}) as sock_info:
+            with server._pool.get_socket() as sock_info:
                 pass
             # When the reaper runs at the same time as the get_socket,
             # maxPoolSize=1 should prevent two sockets from being created.
@@ -546,11 +561,11 @@ class TestClient(IntegrationTest):
             # Assert reaper has removed idle socket and NOT replaced it
             client = rs_or_single_client(maxIdleTimeMS=500)
             server = client._get_topology().select_server(readable_server_selector)
-            with server._pool.get_socket({}) as sock_info_one:
+            with server._pool.get_socket() as sock_info_one:
                 pass
             # Assert that the pool does not close sockets prematurely.
             time.sleep(0.300)
-            with server._pool.get_socket({}) as sock_info_two:
+            with server._pool.get_socket() as sock_info_two:
                 pass
             self.assertIs(sock_info_one, sock_info_two)
             wait_until(
@@ -571,7 +586,7 @@ class TestClient(IntegrationTest):
             wait_until(lambda: 10 == len(server._pool.sockets), "pool initialized with 10 sockets")
 
             # Assert that if a socket is closed, a new one takes its place
-            with server._pool.get_socket({}) as sock_info:
+            with server._pool.get_socket() as sock_info:
                 sock_info.close_socket(None)
             wait_until(
                 lambda: 10 == len(server._pool.sockets),
@@ -584,12 +599,12 @@ class TestClient(IntegrationTest):
         with client_knobs(kill_cursor_frequency=99999999):
             client = rs_or_single_client(maxIdleTimeMS=500)
             server = client._get_topology().select_server(readable_server_selector)
-            with server._pool.get_socket({}) as sock_info:
+            with server._pool.get_socket() as sock_info:
                 pass
             self.assertEqual(1, len(server._pool.sockets))
-            time.sleep(1)  #  Sleep so that the socket becomes stale.
+            time.sleep(1)  # Sleep so that the socket becomes stale.
 
-            with server._pool.get_socket({}) as new_sock_info:
+            with server._pool.get_socket() as new_sock_info:
                 self.assertNotEqual(sock_info, new_sock_info)
             self.assertEqual(1, len(server._pool.sockets))
             self.assertFalse(sock_info in server._pool.sockets)
@@ -598,11 +613,11 @@ class TestClient(IntegrationTest):
             # Test that sockets are reused if maxIdleTimeMS is not set.
             client = rs_or_single_client()
             server = client._get_topology().select_server(readable_server_selector)
-            with server._pool.get_socket({}) as sock_info:
+            with server._pool.get_socket() as sock_info:
                 pass
             self.assertEqual(1, len(server._pool.sockets))
             time.sleep(1)
-            with server._pool.get_socket({}) as new_sock_info:
+            with server._pool.get_socket() as new_sock_info:
                 self.assertEqual(sock_info, new_sock_info)
             self.assertEqual(1, len(server._pool.sockets))
 
@@ -611,7 +626,7 @@ class TestClient(IntegrationTest):
         port are not overloaded.
         """
         host, port = client_context.host, client_context.port
-        kwargs = client_context.default_client_options.copy()
+        kwargs: dict = client_context.default_client_options.copy()
         if client_context.auth_enabled:
             kwargs["username"] = db_user
             kwargs["password"] = db_pwd
@@ -709,9 +724,9 @@ class TestClient(IntegrationTest):
 
     def test_repr(self):
         # Used to test 'eval' below.
-        import bson
+        import bson  # noqa: F401
 
-        client = MongoClient(
+        client = MongoClient(  # type: ignore[type-var]
             "mongodb://localhost:27017,localhost:27018/?replicaSet=replset"
             "&connectTimeoutMS=12345&w=1&wtimeoutms=100",
             connect=False,
@@ -720,9 +735,7 @@ class TestClient(IntegrationTest):
 
         the_repr = repr(client)
         self.assertIn("MongoClient(host=", the_repr)
-        self.assertIn(
-            "document_class=bson.son.SON, " "tz_aware=False, " "connect=False, ", the_repr
-        )
+        self.assertIn("document_class=bson.son.SON, tz_aware=False, connect=False, ", the_repr)
         self.assertIn("connecttimeoutms=12345", the_repr)
         self.assertIn("replicaset='replset'", the_repr)
         self.assertIn("w=1", the_repr)
@@ -741,7 +754,7 @@ class TestClient(IntegrationTest):
         )
         the_repr = repr(client)
         self.assertIn("MongoClient(host=", the_repr)
-        self.assertIn("document_class=dict, " "tz_aware=False, " "connect=False, ", the_repr)
+        self.assertIn("document_class=dict, tz_aware=False, connect=False, ", the_repr)
         self.assertIn("connecttimeoutms=12345", the_repr)
         self.assertIn("replicaset='replset'", the_repr)
         self.assertIn("sockettimeoutms=None", the_repr)
@@ -763,6 +776,7 @@ class TestClient(IntegrationTest):
         for doc in helper_docs:
             self.assertIs(type(doc), dict)
         client = rs_or_single_client(document_class=SON)
+        self.addCleanup(client.close)
         for doc in client.list_databases():
             self.assertIs(type(doc), dict)
 
@@ -983,6 +997,7 @@ class TestClient(IntegrationTest):
         uri = "mongodb://%s" % encoded_socket
         # Confirm we can do operations via the socket.
         client = rs_or_single_client(uri)
+        self.addCleanup(client.close)
         client.pymongo_test.test.insert_one({"dummy": "object"})
         dbs = client.list_database_names()
         self.assertTrue("pymongo_test" in dbs)
@@ -1006,6 +1021,7 @@ class TestClient(IntegrationTest):
         self.assertFalse(isinstance(db.test.find_one(), SON))
 
         c = rs_or_single_client(document_class=SON)
+        self.addCleanup(c.close)
         db = c.pymongo_test
 
         self.assertEqual(SON, c.codec_options.document_class)
@@ -1044,6 +1060,7 @@ class TestClient(IntegrationTest):
         no_timeout = self.client
         timeout_sec = 1
         timeout = rs_or_single_client(socketTimeoutMS=1000 * timeout_sec)
+        self.addCleanup(timeout.close)
 
         no_timeout.pymongo_test.drop_collection("test")
         no_timeout.pymongo_test.test.insert_one({"x": 1})
@@ -1090,14 +1107,16 @@ class TestClient(IntegrationTest):
 
     def test_socketKeepAlive(self):
         pool = get_pool(self.client)
-        with pool.get_socket({}) as sock_info:
+        with pool.get_socket() as sock_info:
             keepalive = sock_info.sock.getsockopt(socket.SOL_SOCKET, socket.SO_KEEPALIVE)
             self.assertTrue(keepalive)
 
+    @no_type_check
     def test_tz_aware(self):
         self.assertRaises(ValueError, MongoClient, tz_aware="foo")
 
         aware = rs_or_single_client(tz_aware=True)
+        self.addCleanup(aware.close)
         naive = self.client
         aware.pymongo_test.drop_collection("test")
 
@@ -1124,9 +1143,10 @@ class TestClient(IntegrationTest):
 
         uri = "mongodb://%s[::1]:%d" % (auth_str, client_context.port)
         if client_context.is_rs:
-            uri += "/?replicaSet=" + client_context.replica_set_name
+            uri += "/?replicaSet=" + (client_context.replica_set_name or "")
 
         client = rs_or_single_client_noauth(uri)
+        self.addCleanup(client.close)
         client.pymongo_test.test.insert_one({"dummy": "object"})
         client.pymongo_test_bernie.test.insert_one({"dummy": "object"})
 
@@ -1225,6 +1245,7 @@ class TestClient(IntegrationTest):
         # to avoid race conditions caused by replica set failover or idle
         # socket reaping.
         client = single_client()
+        self.addCleanup(client.close)
         client.pymongo_test.test.find_one()
         pool = get_pool(client)
         socket_count = len(pool.sockets)
@@ -1248,18 +1269,21 @@ class TestClient(IntegrationTest):
         self.addCleanup(client_context.client.drop_database, "test_lazy_connect_w0")
 
         client = rs_or_single_client(connect=False, w=0)
+        self.addCleanup(client.close)
         client.test_lazy_connect_w0.test.insert_one({})
         wait_until(
             lambda: client.test_lazy_connect_w0.test.count_documents({}) == 1, "find one document"
         )
 
         client = rs_or_single_client(connect=False, w=0)
+        self.addCleanup(client.close)
         client.test_lazy_connect_w0.test.update_one({}, {"$set": {"x": 1}})
         wait_until(
             lambda: client.test_lazy_connect_w0.test.find_one().get("x") == 1, "update one document"
         )
 
         client = rs_or_single_client(connect=False, w=0)
+        self.addCleanup(client.close)
         client.test_lazy_connect_w0.test.delete_one({})
         wait_until(
             lambda: client.test_lazy_connect_w0.test.count_documents({}) == 0, "delete one document"
@@ -1270,6 +1294,7 @@ class TestClient(IntegrationTest):
         # When doing an exhaust query, the socket stays checked out on success
         # but must be checked in on error to avoid semaphore leaks.
         client = rs_or_single_client(maxPoolSize=1, retryReads=False)
+        self.addCleanup(client.close)
         collection = client.pymongo_test.test
         pool = get_pool(client)
         pool._check_interval_seconds = None  # Never check.
@@ -1302,8 +1327,8 @@ class TestClient(IntegrationTest):
         socket_info = one(pool.sockets)
         socket_info.sock.close()
 
-        # SocketInfo.check_auth logs in with the new credential, but gets a
-        # socket.error. Should be reraised as AutoReconnect.
+        # SocketInfo.authenticate logs, but gets a socket.error. Should be
+        # reraised as AutoReconnect.
         self.assertRaises(AutoReconnect, c.test.collection.find_one)
 
         # No semaphore leak, the pool is allowed to make a new socket.
@@ -1336,6 +1361,7 @@ class TestClient(IntegrationTest):
                     None,
                     None,
                     False,
+                    None,
                 ),
                 unpack_res=Cursor(client.pymongo_test.collection)._unpack_response,
                 address=("not-a-member", 27017),
@@ -1363,7 +1389,7 @@ class TestClient(IntegrationTest):
             heartbeat_times.append(time.time())
 
         try:
-            ServerHeartbeatStartedEvent.__init__ = init
+            ServerHeartbeatStartedEvent.__init__ = init  # type: ignore
             listener = HeartbeatStartedListener()
             uri = "mongodb://%s:%d/?heartbeatFrequencyMS=500" % (
                 client_context.host,
@@ -1380,7 +1406,7 @@ class TestClient(IntegrationTest):
 
             client.close()
         finally:
-            ServerHeartbeatStartedEvent.__init__ = old_init
+            ServerHeartbeatStartedEvent.__init__ = old_init  # type: ignore
 
     def test_small_heartbeat_frequency_ms(self):
         uri = "mongodb://example/?heartbeatFrequencyMS=499"
@@ -1506,7 +1532,7 @@ class TestClient(IntegrationTest):
         try:
             while True:
                 for _ in range(10):
-                    client._topology.update_pool(client._MongoClient__all_credentials)
+                    client._topology.update_pool()
                 if generation != pool.gen.get_overall():
                     break
         finally:
@@ -1647,7 +1673,7 @@ class TestClient(IntegrationTest):
         )
         self.assertEqual(client._topology_settings.srv_service_name, "customname")
         client = MongoClient(
-            "mongodb+srv://user:password@test22.test.build.10gen.cc" "/?srvServiceName=customname",
+            "mongodb+srv://user:password@test22.test.build.10gen.cc/?srvServiceName=customname",
             connect=False,
         )
         self.assertEqual(client._topology_settings.srv_service_name, "customname")
@@ -1663,6 +1689,44 @@ class TestClient(IntegrationTest):
         )
         self.assertEqual(len(client.topology_description.server_descriptions()), 2)
 
+    @unittest.skipIf(_HAVE_DNSPYTHON, "dnspython must not be installed")
+    def test_srv_no_dnspython_error(self):
+        with self.assertRaisesRegex(ConfigurationError, 'The "dnspython" module must be'):
+            MongoClient("mongodb+srv://test1.test.build.10gen.cc/")
+
+    @unittest.skipIf(
+        client_context.load_balancer or client_context.serverless,
+        "loadBalanced clients do not run SDAM",
+    )
+    @unittest.skipIf(sys.platform == "win32", "Windows does not support SIGSTOP")
+    def test_sigstop_sigcont(self):
+        test_dir = os.path.dirname(os.path.realpath(__file__))
+        script = os.path.join(test_dir, "sigstop_sigcont.py")
+        p = subprocess.Popen(
+            [sys.executable, script, client_context.uri],
+            stdin=subprocess.PIPE,
+            stdout=subprocess.PIPE,
+            stderr=subprocess.STDOUT,
+        )
+        self.addCleanup(p.wait, timeout=1)
+        self.addCleanup(p.kill)
+        time.sleep(1)
+        # Stop the child, sleep for twice the streaming timeout
+        # (heartbeatFrequencyMS + connectTimeoutMS), and restart.
+        os.kill(p.pid, signal.SIGSTOP)
+        time.sleep(2)
+        os.kill(p.pid, signal.SIGCONT)
+        time.sleep(0.5)
+        # Tell the script to exit gracefully.
+        outs, _ = p.communicate(input=b"q\n", timeout=10)
+        self.assertTrue(outs)
+        log_output = outs.decode("utf-8")
+        self.assertIn("TEST STARTED", log_output)
+        self.assertIn("ServerHeartbeatStartedEvent", log_output)
+        self.assertIn("ServerHeartbeatSucceededEvent", log_output)
+        self.assertIn("TEST COMPLETED", log_output)
+        self.assertNotIn("ServerHeartbeatFailedEvent", log_output)
+
 
 class TestExhaustCursor(IntegrationTest):
     """Test that clients properly handle errors from exhaust cursors."""
@@ -1826,7 +1890,7 @@ class TestClientLazyConnect(IntegrationTest):
         lazy_client_trial(reset, delete_one, test, self._get_client)
 
     def test_find_one(self):
-        results = []
+        results: list = []
 
         def reset(collection):
             collection.drop()
@@ -1860,7 +1924,7 @@ class TestMongoClientFailover(MockClientTest):
         # Fail over.
         c.kill_host("a:1")
         c.mock_primary = "b:2"
-        wait_until(lambda: c.address == ("b", 2), "wait for server " "address to be " "updated")
+        wait_until(lambda: c.address == ("b", 2), "wait for server address to be updated")
         # a:1 not longer in nodes.
         self.assertLess(len(c.nodes), 3)
 
