@@ -14,21 +14,18 @@
 
 """Test the database module."""
 
-import datetime
 import re
 import sys
+from typing import Any, Iterable, List, Mapping
 
 sys.path[0:0] = [""]
 
-from test import IntegrationTest, SkipTest, client_context, unittest
+from test import IntegrationTest, client_context, unittest
 from test.test_custom_types import DECIMAL_CODECOPTS
 from test.utils import (
     IMPOSSIBLE_WRITE_CONCERN,
-    DeprecationFilter,
     OvertCommandListener,
-    ignore_deprecations,
     rs_or_single_client,
-    server_started_with_auth,
     wait_until,
 )
 
@@ -43,7 +40,6 @@ from pymongo.collection import Collection
 from pymongo.database import Database
 from pymongo.errors import (
     CollectionInvalid,
-    ConfigurationError,
     ExecutionTimeout,
     InvalidName,
     OperationFailure,
@@ -58,6 +54,8 @@ from pymongo.write_concern import WriteConcern
 class TestDatabaseNoConnect(unittest.TestCase):
     """Test Database features on a client that does not connect."""
 
+    client: MongoClient
+
     @classmethod
     def setUpClass(cls):
         cls.client = MongoClient(connect=False)
@@ -96,7 +94,26 @@ class TestDatabaseNoConnect(unittest.TestCase):
         self.assertIn("has no attribute '_does_not_exist'", str(context.exception))
 
     def test_iteration(self):
-        self.assertRaises(TypeError, next, self.client.pymongo_test)
+        db = self.client.pymongo_test
+        if "PyPy" in sys.version:
+            msg = "'NoneType' object is not callable"
+        else:
+            msg = "'Database' object is not iterable"
+        # Iteration fails
+        with self.assertRaisesRegex(TypeError, msg):
+            for _ in db:  # type: ignore[misc] # error: "None" not callable  [misc]
+                break
+        # Index fails
+        with self.assertRaises(TypeError):
+            _ = db[0]
+        # next fails
+        with self.assertRaisesRegex(TypeError, "'Database' object is not iterable"):
+            _ = next(db)
+        # .next() fails
+        with self.assertRaisesRegex(TypeError, "'Database' object is not iterable"):
+            _ = db.next()
+        # Do not implement typing.Iterable.
+        self.assertNotIsInstance(db, Iterable)
 
 
 class TestDatabase(IntegrationTest):
@@ -138,7 +155,7 @@ class TestDatabase(IntegrationTest):
         test = db.create_collection("test")
         self.assertTrue("test" in db.list_collection_names())
         test.insert_one({"hello": "world"})
-        self.assertEqual(db.test.find_one()["hello"], "world")
+        self.assertEqual(db.test.find_one()["hello"], "world")  # type: ignore
 
         db.drop_collection("test.foo")
         db.create_collection("test.foo")
@@ -193,6 +210,7 @@ class TestDatabase(IntegrationTest):
             self.assertNotIn("nameOnly", results["started"][0].command)
 
         # Should send nameOnly (except on 2.6).
+        filter: Any
         for filter in (None, {}, {"name": {"$in": ["capped", "non_capped"]}}):
             results.clear()
             names = db.list_collection_names(filter=filter)
@@ -220,7 +238,7 @@ class TestDatabase(IntegrationTest):
             self.assertTrue("$" not in coll)
 
         # Duplicate check.
-        coll_cnt = {}
+        coll_cnt: dict = {}
         for coll in colls:
             try:
                 # Found duplicate.
@@ -228,7 +246,7 @@ class TestDatabase(IntegrationTest):
                 self.assertTrue(False)
             except KeyError:
                 coll_cnt[coll] = 1
-        coll_cnt = {}
+        coll_cnt: dict = {}
 
         # Checking if is there any collection which don't exists.
         if (
@@ -461,6 +479,7 @@ class TestDatabase(IntegrationTest):
         self.assertEqual(None, db.test.find_one({"hello": "test"}))
 
         b = db.test.find_one()
+        assert b is not None
         b["hello"] = "mike"
         db.test.replace_one({"_id": b["_id"]}, b)
 
@@ -477,12 +496,12 @@ class TestDatabase(IntegrationTest):
         db = self.client.pymongo_test
         db.test.drop()
         db.test.insert_one({"x": 9223372036854775807})
-        retrieved = db.test.find_one()["x"]
+        retrieved = db.test.find_one()["x"]  # type: ignore
         self.assertEqual(Int64(9223372036854775807), retrieved)
         self.assertIsInstance(retrieved, Int64)
         db.test.delete_many({})
         db.test.insert_one({"x": Int64(1)})
-        retrieved = db.test.find_one()["x"]
+        retrieved = db.test.find_one()["x"]  # type: ignore
         self.assertEqual(Int64(1), retrieved)
         self.assertIsInstance(retrieved, Int64)
 
@@ -504,8 +523,8 @@ class TestDatabase(IntegrationTest):
             length += 1
         self.assertEqual(length, 2)
 
-        db.test.delete_one(db.test.find_one())
-        db.test.delete_one(db.test.find_one())
+        db.test.delete_one(db.test.find_one())  # type: ignore[arg-type]
+        db.test.delete_one(db.test.find_one())  # type: ignore[arg-type]
         self.assertEqual(db.test.find_one(), None)
 
         db.test.insert_one({"x": 1})
@@ -625,14 +644,14 @@ class TestDatabase(IntegrationTest):
             "write_concern": WriteConcern(w=1),
             "read_concern": ReadConcern(level="local"),
         }
-        db2 = db1.with_options(**newopts)
+        db2 = db1.with_options(**newopts)  # type: ignore[arg-type]
         for opt in newopts:
             self.assertEqual(getattr(db2, opt), newopts.get(opt, getattr(db1, opt)))
 
 
 class TestDatabaseAggregation(IntegrationTest):
     def setUp(self):
-        self.pipeline = [
+        self.pipeline: List[Mapping[str, Any]] = [
             {"$listLocalSessions": {}},
             {"$limit": 1},
             {"$addFields": {"dummy": "dummy field"}},
@@ -649,6 +668,7 @@ class TestDatabaseAggregation(IntegrationTest):
     @client_context.require_no_mongos
     def test_database_aggregation_fake_cursor(self):
         coll_name = "test_output"
+        write_stage: dict
         if client_context.version < (4, 3):
             db_name = "admin"
             write_stage = {"$out": coll_name}
