@@ -22,15 +22,17 @@ import time
 sys.path[0:0] = [""]
 
 from test import IntegrationTest, unittest
+from test.pymongo_mocks import DummyMonitor
 from test.utils import (
-    Barrier,
+    CMAPListener,
+    HeartbeatEventListener,
     TestCreator,
     assertion_context,
-    cdecimal_patched,
     client_context,
     get_pool,
     rs_or_single_client,
     server_name_to_type,
+    single_client,
     wait_until,
 )
 from test.utils_spec_runner import SpecRunner, SpecRunnerThread
@@ -44,7 +46,7 @@ from pymongo.errors import (
     NotPrimaryError,
     OperationFailure,
 )
-from pymongo.hello import Hello
+from pymongo.hello import Hello, HelloCompat
 from pymongo.helpers import _check_command_response, _check_write_command_response
 from pymongo.server_description import SERVER_TYPE, ServerDescription
 from pymongo.settings import TopologySettings
@@ -56,27 +58,7 @@ from pymongo.uri_parser import parse_uri
 _TEST_PATH = os.path.join(os.path.dirname(os.path.realpath(__file__)), "discovery_and_monitoring")
 
 
-class MockMonitor(object):
-    def __init__(self, server_description, topology, pool, topology_settings):
-        self._server_description = server_description
-
-    def cancel_check(self):
-        pass
-
-    def open(self):
-        pass
-
-    def close(self):
-        pass
-
-    def join(self):
-        pass
-
-    def request_check(self):
-        pass
-
-
-def create_mock_topology(uri, monitor_class=MockMonitor):
+def create_mock_topology(uri, monitor_class=DummyMonitor):
     parsed_uri = parse_uri(uri)
     replica_set_name = None
     direct_connection = None
@@ -210,6 +192,9 @@ def check_outcome(self, topology, outcome):
         topology_type_name(topology.description.topology_type),
     )
 
+    self.assertEqual(outcome.get("maxSetVersion"), topology.description.max_set_version)
+    self.assertEqual(outcome.get("maxElectionId"), topology.description.max_election_id)
+
 
 def create_test(scenario_def):
     def run_scenario(self):
@@ -280,7 +265,7 @@ class TestClusterTimeComparison(unittest.TestCase):
 class TestIgnoreStaleErrors(IntegrationTest):
     def test_ignore_stale_connection_errors(self):
         N_THREADS = 5
-        barrier = Barrier(N_THREADS, timeout=30)
+        barrier = threading.Barrier(N_THREADS, timeout=30)
         client = rs_or_single_client(minPoolSize=N_THREADS)
         self.addCleanup(client.close)
 
@@ -301,7 +286,7 @@ class TestIgnoreStaleErrors(IntegrationTest):
         def insert_command(i):
             try:
                 client.test.command("insert", "test", documents=[{"i": i}])
-            except AutoReconnect as exc:
+            except AutoReconnect:
                 pass
 
         threads = []
@@ -319,6 +304,42 @@ class TestIgnoreStaleErrors(IntegrationTest):
         client.admin.command("ping")
 
 
+class CMAPHeartbeatListener(HeartbeatEventListener, CMAPListener):
+    pass
+
+
+class TestPoolManagement(IntegrationTest):
+    @client_context.require_failCommand_appName
+    def test_pool_unpause(self):
+        # This test implements the prose test "Connection Pool Management"
+        listener = CMAPHeartbeatListener()
+        client = single_client(
+            appName="SDAMPoolManagementTest", heartbeatFrequencyMS=500, event_listeners=[listener]
+        )
+        self.addCleanup(client.close)
+        # Assert that ConnectionPoolReadyEvent occurs after the first
+        # ServerHeartbeatSucceededEvent.
+        listener.wait_for_event(monitoring.PoolReadyEvent, 1)
+        pool_ready = listener.events_by_type(monitoring.PoolReadyEvent)[0]
+        hb_succeeded = listener.events_by_type(monitoring.ServerHeartbeatSucceededEvent)[0]
+        self.assertGreater(listener.events.index(pool_ready), listener.events.index(hb_succeeded))
+
+        listener.reset()
+        fail_hello = {
+            "mode": {"times": 2},
+            "data": {
+                "failCommands": [HelloCompat.LEGACY_CMD, "hello"],
+                "errorCode": 1234,
+                "appName": "SDAMPoolManagementTest",
+            },
+        }
+        with self.fail_point(fail_hello):
+            listener.wait_for_event(monitoring.ServerHeartbeatFailedEvent, 1)
+            listener.wait_for_event(monitoring.PoolClearedEvent, 1)
+            listener.wait_for_event(monitoring.ServerHeartbeatSucceededEvent, 1)
+            listener.wait_for_event(monitoring.PoolReadyEvent, 1)
+
+
 class TestIntegration(SpecRunner):
     # Location of JSON test specifications.
     TEST_PATH = os.path.join(
@@ -334,20 +355,14 @@ class TestIntegration(SpecRunner):
                     and not e.new_description.is_server_type_known
                 )
 
+            assert self.server_listener is not None
             return len(self.server_listener.matching(marked_unknown))
         # Only support CMAP events for now.
         self.assertTrue(event.startswith("Pool") or event.startswith("Conn"))
         event_type = getattr(monitoring, event)
+        assert self.pool_listener is not None
         return self.pool_listener.event_count(event_type)
 
-    def maybe_skip_scenario(self, test):
-        """Override to skip threaded tests when cdecimal is installed on 2.7"""
-        super(TestIntegration, self).maybe_skip_scenario(test)
-        # PYTHON-2332
-        ops = [op["name"] for op in test["operations"]]
-        if cdecimal_patched() and "startThread" in ops:
-            raise unittest.SkipTest("PYTHON-2332 test fails with cdecimal")
-
     def assert_event_count(self, event, count):
         """Run the assertEventCount test operation.
 
