@@ -23,6 +23,8 @@ import sys
 import zipfile
 from io import BytesIO
 
+from pymongo.database import Database
+
 sys.path[0:0] = [""]
 
 from test import IntegrationTest, qcheck, unittest
@@ -47,6 +49,8 @@ from pymongo.message import _CursorAddress
 class TestGridFileNoConnect(unittest.TestCase):
     """Test GridFile features on a client that does not connect."""
 
+    db: Database
+
     @classmethod
     def setUpClass(cls):
         cls.db = MongoClient(connect=False).pymongo_test
@@ -253,7 +257,7 @@ class TestGridFile(IntegrationTest):
         cursor_dict.pop("_Cursor__session")
         cursor_clone_dict = cursor_clone.__dict__.copy()
         cursor_clone_dict.pop("_Cursor__session")
-        self.assertEqual(cursor_dict, cursor_clone_dict)
+        self.assertDictEqual(cursor_dict, cursor_clone_dict)
 
         self.assertRaises(NotImplementedError, cursor.add_option, 0)
         self.assertRaises(NotImplementedError, cursor.remove_option, 0)
@@ -576,7 +580,7 @@ Bye"""
         self.assertEqual([b"hello world"], list(g))
 
     def test_read_unaligned_buffer_size(self):
-        in_data = b"This is a text that doesn't " b"quite fit in a single 16-byte chunk."
+        in_data = b"This is a text that doesn't quite fit in a single 16-byte chunk."
         f = GridIn(self.db.fs, chunkSize=16)
         f.write(in_data)
         f.close()
