@@ -15,22 +15,31 @@
 """Test the client_session module."""
 
 import copy
-import os
 import sys
 import time
 from io import BytesIO
+from typing import Any, Callable, List, Set, Tuple
+
+from pymongo.mongo_client import MongoClient
 
 sys.path[0:0] = [""]
 
 from test import IntegrationTest, SkipTest, client_context, unittest
-from test.utils import EventListener, TestCreator, rs_or_single_client, wait_until
-from test.utils_spec_runner import SpecRunner
+from test.utils import (
+    EventListener,
+    ExceptionCatchingThread,
+    rs_or_single_client,
+    wait_until,
+)
 
 from bson import DBRef
 from gridfs import GridFS, GridFSBucket
 from pymongo import ASCENDING, IndexModel, InsertOne, monitoring
+from pymongo.command_cursor import CommandCursor
 from pymongo.common import _MAX_END_SESSIONS
+from pymongo.cursor import Cursor
 from pymongo.errors import ConfigurationError, InvalidOperation, OperationFailure
+from pymongo.operations import UpdateOne
 from pymongo.read_concern import ReadConcern
 
 
@@ -59,6 +68,9 @@ def session_ids(client):
 
 
 class TestSession(IntegrationTest):
+    client2: MongoClient
+    sensitive_commands: Set[str]
+
     @classmethod
     @client_context.require_sessions
     def setUpClass(cls):
@@ -139,7 +151,7 @@ class TestSession(IntegrationTest):
                 kw = copy.copy(kw)
                 kw["session"] = s
                 with self.assertRaisesRegex(
-                    InvalidOperation, "Can only use session with the MongoClient" " that started it"
+                    InvalidOperation, "Can only use session with the MongoClient that started it"
                 ):
                     f(*args, **kw)
 
@@ -167,6 +179,61 @@ class TestSession(IntegrationTest):
                         "%s did not return implicit session to pool" % (f.__name__,),
                     )
 
+    def test_implicit_sessions_checkout(self):
+        # "To confirm that implicit sessions only allocate their server session after a
+        # successful connection checkout" test from Driver Sessions Spec.
+        succeeded = False
+        lsid_set = set()
+        failures = 0
+        for _ in range(5):
+            listener = EventListener()
+            client = rs_or_single_client(event_listeners=[listener], maxPoolSize=1)
+            cursor = client.db.test.find({})
+            ops: List[Tuple[Callable, List[Any]]] = [
+                (client.db.test.find_one, [{"_id": 1}]),
+                (client.db.test.delete_one, [{}]),
+                (client.db.test.update_one, [{}, {"$set": {"x": 2}}]),
+                (client.db.test.bulk_write, [[UpdateOne({}, {"$set": {"x": 2}})]]),
+                (client.db.test.find_one_and_delete, [{}]),
+                (client.db.test.find_one_and_update, [{}, {"$set": {"x": 1}}]),
+                (client.db.test.find_one_and_replace, [{}, {}]),
+                (client.db.test.aggregate, [[{"$limit": 1}]]),
+                (client.db.test.find, []),
+                (client.server_info, [{}]),
+                (client.db.aggregate, [[{"$listLocalSessions": {}}, {"$limit": 1}]]),
+                (cursor.distinct, ["_id"]),
+                (client.db.list_collections, []),
+            ]
+            threads = []
+            listener.results.clear()
+
+            def thread_target(op, *args):
+                res = op(*args)
+                if isinstance(res, (Cursor, CommandCursor)):
+                    list(res)
+
+            for op, args in ops:
+                threads.append(
+                    ExceptionCatchingThread(
+                        target=thread_target, args=[op, *args], name=op.__name__
+                    )
+                )
+                threads[-1].start()
+            self.assertEqual(len(threads), len(ops))
+            for thread in threads:
+                thread.join()
+                self.assertIsNone(thread.exc)
+            client.close()
+            lsid_set.clear()
+            for i in listener.results["started"]:
+                if i.command.get("lsid"):
+                    lsid_set.add(i.command.get("lsid")["id"])
+            if len(lsid_set) == 1:
+                succeeded = True
+            else:
+                failures += 1
+        self.assertTrue(succeeded, lsid_set)
+
     def test_pool_lifo(self):
         # "Pool is LIFO" test from Driver Sessions Spec.
         a = self.client.start_session()
@@ -223,7 +290,7 @@ class TestSession(IntegrationTest):
 
     def test_client(self):
         client = self.client
-        ops = [
+        ops: list = [
             (client.server_info, [], {}),
             (client.list_database_names, [], {}),
             (client.drop_database, ["pymongo_test"], {}),
@@ -234,7 +301,7 @@ class TestSession(IntegrationTest):
     def test_database(self):
         client = self.client
         db = client.pymongo_test
-        ops = [
+        ops: list = [
             (db.command, ["ping"], {}),
             (db.create_collection, ["collection"], {}),
             (db.list_collection_names, [], {}),
@@ -484,6 +551,7 @@ class TestSession(IntegrationTest):
         # Explicit session.
         with client.start_session() as s:
             cursor = bucket.find(session=s)
+            assert cursor.session is not None
             s = cursor.session
             files = list(cursor)
             cursor.__del__()
@@ -671,7 +739,7 @@ class TestSession(IntegrationTest):
         self.addCleanup(client.close)
         db = client.pymongo_test
         coll = db.test_unacked_writes
-        ops = [
+        ops: list = [
             (client.drop_database, [db.name], {}),
             (db.create_collection, ["collection"], {}),
             (db.drop_collection, ["collection"], {}),
@@ -710,6 +778,9 @@ class TestSession(IntegrationTest):
 
 
 class TestCausalConsistency(unittest.TestCase):
+    listener: SessionTestListener
+    client: MongoClient
+
     @classmethod
     def setUpClass(cls):
         cls.listener = SessionTestListener()
@@ -764,6 +835,8 @@ class TestCausalConsistency(unittest.TestCase):
                 self.assertRaises(ValueError, sess2.advance_cluster_time, {})
                 self.assertRaises(TypeError, sess2.advance_operation_time, 1)
                 # No error
+                assert sess.cluster_time is not None
+                assert sess.operation_time is not None
                 sess2.advance_cluster_time(sess.cluster_time)
                 sess2.advance_operation_time(sess.operation_time)
                 self.assertEqual(sess.cluster_time, sess2.cluster_time)
@@ -1083,54 +1156,5 @@ class TestClusterTime(IntegrationTest):
                     )
 
 
-class TestSpec(SpecRunner):
-    RUN_ON_SERVERLESS = True
-    # Location of JSON test specifications.
-    TEST_PATH = os.path.join(os.path.dirname(os.path.realpath(__file__)), "sessions", "legacy")
-
-    def last_two_command_events(self):
-        """Return the last two command started events."""
-        started_events = self.listener.results["started"][-2:]
-        self.assertEqual(2, len(started_events))
-        return started_events
-
-    def assert_same_lsid_on_last_two_commands(self):
-        """Run the assertSameLsidOnLastTwoCommands test operation."""
-        event1, event2 = self.last_two_command_events()
-        self.assertEqual(event1.command["lsid"], event2.command["lsid"])
-
-    def assert_different_lsid_on_last_two_commands(self):
-        """Run the assertDifferentLsidOnLastTwoCommands test operation."""
-        event1, event2 = self.last_two_command_events()
-        self.assertNotEqual(event1.command["lsid"], event2.command["lsid"])
-
-    def assert_session_dirty(self, session):
-        """Run the assertSessionDirty test operation.
-
-        Assert that the given session is dirty.
-        """
-        self.assertIsNotNone(session._server_session)
-        self.assertTrue(session._server_session.dirty)
-
-    def assert_session_not_dirty(self, session):
-        """Run the assertSessionNotDirty test operation.
-
-        Assert that the given session is not dirty.
-        """
-        self.assertIsNotNone(session._server_session)
-        self.assertFalse(session._server_session.dirty)
-
-
-def create_test(scenario_def, test, name):
-    @client_context.require_test_commands
-    def run_scenario(self):
-        self.run_scenario(scenario_def, test)
-
-    return run_scenario
-
-
-test_creator = TestCreator(create_test, TestSpec, TestSpec.TEST_PATH)
-test_creator.create_tests()
-
 if __name__ == "__main__":
     unittest.main()
