@@ -16,6 +16,7 @@
 
 import os
 import sys
+from io import BytesIO
 
 sys.path[0:0] = [""]
 
@@ -29,10 +30,6 @@ from test.utils import (
 )
 from test.utils_spec_runner import SpecRunner
 
-from bson.py3compat import StringIO
-from bson import encode
-from bson.raw_bson import RawBSONDocument
-
 from gridfs import GridFS, GridFSBucket
 from pymongo import WriteConcern, client_session
 from pymongo.client_session import TransactionOptions
@@ -101,17 +98,17 @@ class TestTransactions(TransactionsBase):
             max_commit_time_ms=10000,
         )
         with self.assertRaisesRegex(TypeError, "read_concern must be "):
-            TransactionOptions(read_concern={})
+            TransactionOptions(read_concern={})  # type: ignore
         with self.assertRaisesRegex(TypeError, "write_concern must be "):
-            TransactionOptions(write_concern={})
+            TransactionOptions(write_concern={})  # type: ignore
         with self.assertRaisesRegex(
             ConfigurationError, "transactions do not support unacknowledged write concern"
         ):
             TransactionOptions(write_concern=WriteConcern(w=0))
         with self.assertRaisesRegex(TypeError, "is not valid for read_preference"):
-            TransactionOptions(read_preference={})
+            TransactionOptions(read_preference={})  # type: ignore
         with self.assertRaisesRegex(TypeError, "max_commit_time_ms must be an integer or None"):
-            TransactionOptions(max_commit_time_ms="10000")
+            TransactionOptions(max_commit_time_ms="10000")  # type: ignore
 
     @client_context.require_transactions
     def test_transaction_write_concern_override(self):
@@ -135,11 +132,10 @@ class TestTransactions(TransactionsBase):
                 coll.find_one_and_replace({}, {}, session=s)
                 coll.find_one_and_update({}, {"$set": {"a": 1}}, session=s)
 
-        unsupported_txn_writes = [
+        unsupported_txn_writes: list = [
             (client.drop_database, [db.name], {}),
             (db.drop_collection, ["collection"], {}),
             (coll.drop, [], {}),
-            (coll.map_reduce, ["function() {}", "function() {}", "output"], {}),
             (coll.rename, ["collection2"], {}),
             # Drop collection2 between tests of "rename", above.
             (coll.database.drop_collection, ["collection2"], {}),
@@ -284,14 +280,14 @@ class TestTransactions(TransactionsBase):
                 bucket.download_to_stream,
                 (
                     1,
-                    StringIO(),
+                    BytesIO(),
                 ),
             ),
             (
                 bucket.download_to_stream_by_name,
                 (
                     "name",
-                    StringIO(),
+                    BytesIO(),
                 ),
             ),
             (bucket.delete, (1,)),
@@ -313,12 +309,11 @@ class TestTransactions(TransactionsBase):
                     InvalidOperation,
                     "GridFS does not support multi-document transactions",
                 ):
-                    op(*args, session=s)
+                    op(*args, session=s)  # type: ignore
 
     # Require 4.2+ for large (16MB+) transactions.
     @client_context.require_version_min(4, 2)
     @client_context.require_transactions
-    @unittest.skipIf(sys.platform.startswith("java"), "Jython is too slow to pass this test")
     @unittest.skipIf(sys.platform == "win32", "Our Windows machines are too slow to pass this test")
     def test_transaction_starts_with_batched_write(self):
         if "PyPy" in sys.version and client_context.tls:
@@ -335,14 +330,14 @@ class TestTransactions(TransactionsBase):
         listener.reset()
         self.addCleanup(client.close)
         self.addCleanup(coll.drop)
-        large_str = "\0" * (1 * 1024 * 1024)
-        ops = [InsertOne(RawBSONDocument(encode({"a": large_str}))) for _ in range(48)]
+        large_str = "\0" * (10 * 1024 * 1024)
+        ops = [InsertOne({"a": large_str}) for _ in range(10)]
         with client.start_session() as session:
             with session.start_transaction():
                 coll.bulk_write(ops, session=session)
         # Assert commands were constructed properly.
         self.assertEqual(
-            ["insert", "insert", "commitTransaction"], listener.started_command_names()
+            ["insert", "insert", "insert", "commitTransaction"], listener.started_command_names()
         )
         first_cmd = listener.results["started"][0].command
         self.assertTrue(first_cmd["startTransaction"])
@@ -352,7 +347,7 @@ class TestTransactions(TransactionsBase):
             self.assertNotIn("startTransaction", event.command)
             self.assertEqual(lsid, event.command["lsid"])
             self.assertEqual(txn_number, event.command["txnNumber"])
-        self.assertEqual(48, coll.count_documents({}))
+        self.assertEqual(10, coll.count_documents({}))
 
 
 class PatchSessionTimeout(object):
@@ -397,12 +392,12 @@ class TestTransactionsConvenientAPI(TransactionsBase):
 
         self.db.test.insert_one({})
 
-        def callback(session):
+        def callback2(session):
             self.db.test.insert_one({}, session=session)
             return "Foo"
 
         with self.client.start_session() as s:
-            self.assertEqual(s.with_transaction(callback), "Foo")
+            self.assertEqual(s.with_transaction(callback2), "Foo")
 
     @client_context.require_transactions
     def test_callback_not_retried_after_timeout(self):
@@ -413,7 +408,7 @@ class TestTransactionsConvenientAPI(TransactionsBase):
 
         def callback(session):
             coll.insert_one({}, session=session)
-            err = {
+            err: dict = {
                 "ok": 0,
                 "errmsg": "Transaction 7819 has been aborted.",
                 "code": 251,
