@@ -21,6 +21,7 @@ from test import IntegrationTest, client_context, client_knobs
 from test.utils import (
     CMAPListener,
     CompareType,
+    EventListener,
     OvertCommandListener,
     ServerAndTopologyEventListener,
     camel_to_snake,
@@ -29,6 +30,7 @@ from test.utils import (
     prepare_spec_arguments,
     rs_client,
 )
+from typing import List
 
 from bson import decode, encode
 from bson.binary import Binary
@@ -80,6 +82,10 @@ class SpecRunnerThread(threading.Thread):
 
 
 class SpecRunner(IntegrationTest):
+    mongos_clients: List
+    knobs: client_knobs
+    listener: EventListener
+
     @classmethod
     def setUpClass(cls):
         super(SpecRunner, cls).setUpClass()
@@ -97,7 +103,7 @@ class SpecRunner(IntegrationTest):
     def setUp(self):
         super(SpecRunner, self).setUp()
         self.targets = {}
-        self.listener = None
+        self.listener = None  # type: ignore
         self.pool_listener = None
         self.server_listener = None
         self.maxDiff = None
@@ -209,6 +215,7 @@ class SpecRunner(IntegrationTest):
                         ids = expected_result[res]
                         if isinstance(ids, dict):
                             ids = [ids[str(i)] for i in range(len(ids))]
+
                         self.assertEqual(ids, result.inserted_ids, prop)
                 elif prop == "upserted_ids":
                     # Convert indexes from strings to integers.
